table = [
    {
        "episode_number": 1,
        "episode_title": "Land of Visible Pain",
        "file_extension": "mkv",
        "file_name": "01 - Land of Visible Pain.mkv"
    },
    {
        "anime_title": "37 [Ruberia] Death Note",
        "episode_number": 37,
        "file_checksum": "6FA7D273",
        "file_extension": "avi",
        "file_name": "37 [Ruberia]_Death_Note_-_37v2_[FINAL]_[XviD][6FA7D273].avi",
        "release_information": "FINAL",
        "release_version": 2,
        "video_term": "XviD"
    },
    {
        "anime_title": "5 centimeters per second",
        "audio_term": "flac",
        "file_extension": "mkv",
        "file_name": "5_centimeters_per_second[1904x1072.h264.flac][niizk].mkv",
        "release_group": "niizk",
        "video_resolution": "1904x1072",
        "video_term": "h264"
    },
    {
        "anime_season": 1,
        "anime_title": "After War Gundam X",
        "episode_number": 3,
        "episode_title": "My Mount is Fierce!",
        "file_extension": "mkv",
        "file_name": "After War Gundam X - 1x03 - My Mount is Fierce!.mkv"
    },
    {
        "anime_title": "Aim For The Top! Gunbuster",
        "audio_term": "FLAC",
        "episode_number": 1,
        "episode_prefix": "ep",
        "file_checksum": "69ECCDCF",
        "file_extension": "mkv",
        "file_name": "Aim_For_The_Top!_Gunbuster-ep1.BD(H264.FLAC.10bit)[KAA][69ECCDCF].mkv",
        "release_group": "KAA",
        "source": "BD",
        "video_term": [
            "H264",
            "10bit"
        ]
    },
    {
        "anime_title": "Attack on Titan",
        "anime_season": 3,
        "anime_season_prefix": "Season",
        "file_name": "Attack on Titan Season 3"
    },
    {
        "anime_title": "Attack on Titan",
        "episode_number": 3,
        "episode_prefix": "Episode",
        "episode_title": "A Dim Light Amid Despair / Humanity's Comeback, Part 1",
        "file_name": "Attack on Titan - Episode 3 - A Dim Light Amid Despair / Humanity's Comeback, Part 1"
    },
    {
        "anime_title": "Bakemonogatari",
        "audio_term": "AACx2",
        "episode_number": 1,
        "file_extension": "mp4",
        "file_name": "Bakemonogatari - 01 (BD 1280x720 AVC AACx2).mp4",
        "source": "BD",
        "video_resolution": "1280x720",
        "video_term": "AVC"
    },
    {
        "anime_title": "Byousoku 5 Centimeter",
        "audio_term": [
            "2.0ch",
            "AAC"
        ],
        "file_name": "Byousoku 5 Centimeter [Blu-Ray][1920x1080 H.264][2.0ch AAC][SOFTSUBS]",
        "source": "Blu-Ray",
        "subtitles": "SOFTSUBS",
        "video_resolution": "1920x1080",
        "video_term": "H.264"
    },
    {
        "anime_title": "Chrono Crusade",
        "episode_number": [
            1,
            5
        ],
        "episode_prefix": "ep.",
        "file_name": "Chrono Crusade ep. 1-5"
    },
    {
        "anime_title": "Code Geass R2 TV",
        "episode_number": 20,
        "episode_total": 25,
        "file_extension": "mkv",
        "file_name": "Code_Geass_R2_TV_[20_of_25]_[ru_jp]_[HDTV]_[Varies_&_Cuba77_&_AnimeReactor_RU].mkv",
        "other": [
            "Varies",
            "&",
            "Cuba77",
            "&",
            "AnimeReactor",
            "RU"
        ],
        "release_group": "ru_jp",
        "source": "HDTV"
    },
    {
        "anime_title": "Cyborg 009",
        "anime_year": 1968,
        "episode_number": 6,
        "episode_prefix": "episode",
        "file_checksum": "30C15D62",
        "file_extension": "mp4",
        "file_name": "Cyborg 009 (1968) [TSHS] episode 06 [30C15D62].mp4",
        "release_group": "TSHS"
    },
    {
        "anime_title": "DRAMAtical Murder",
        "episode_number": 1,
        "episode_prefix": "Episode",
        "episode_title": "Data 01 Login",
        "file_name": "DRAMAtical Murder Episode 1 - Data_01_Login"
    },
    {
        "anime_title": "Detective Conan",
        "episode_number": [
            316,
            317
        ],
        "file_checksum": "2411959B",
        "file_extension": "mkv",
        "file_name": "Detective Conan - 316-317 [DCTP][2411959B].mkv",
        "release_group": "DCTP"
    },
    {
        "anime_title": "Dragon Ball KAI",
        "episode_number": 1,
        "file_extension": "mkv",
        "file_name": "Dragon.Ball.KAI.-.01.-.1080p.BluRay.x264.DHD.mkv",
        "other": "DHD",
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Dragon Ball Z Movies",
        "audio_term": "DTS",
        "episode_number": [
            8,
            10
        ],
        "file_name": "Dragon_Ball_Z_Movies_8_&_10_[720p,BluRay,DTS,x264]_-_THORA",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "720p",
        "video_term": "x264"
    },
    {
        "episode_number": 1,
        "episode_prefix": "Ep.",
        "episode_title": "The Boy in the Iceberg",
        "file_name": "Ep. 01 - The Boy in the Iceberg"
    },
    {
        "anime_title": "Evangelion Shin Gekijouban Q",
        "audio_term": [
            "FLACx2",
            "5.1ch"
        ],
        "file_extension": "mkv",
        "file_name": "Evangelion Shin Gekijouban Q (BDrip 1920x1080 x264 FLACx2 5.1ch)-ank.mkv",
        "other": "ank",
        "source": "BDrip",
        "video_resolution": "1920x1080",
        "video_term": "x264"
    },
    {
        "anime_title": "Evangelion The New Movie Q",
        "audio_term": "AACx2",
        "file_extension": "mp4",
        "file_name": "Evangelion The New Movie Q (BD 1280x720 AVC AACx2 [5.1+2.0]).mp4",
        "other": "[5.1+2.0]",
        "source": "BD",
        "video_resolution": "1280x720",
        "video_term": "AVC"
    },
    {
        "anime_title": "Evangelion 1.0 You Are [Not] Alone",
        "file_name": "Evangelion_1.0_You_Are_[Not]_Alone_(1080p)_[@Home]",
        "release_group": "@Home",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Evangelion 1.11 You Are (Not) Alone",
        "anime_year": 2009,
        "audio_term": "DTS-ES",
        "file_extension": "mkv",
        "file_name": "Evangelion_1.11_You_Are_(Not)_Alone_(2009)_[1080p,BluRay,x264,DTS-ES]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Evangelion 1.11 You Are (Not) Alone",
        "audio_term": "DTS-ES",
        "file_extension": "mkv",
        "file_name": "Evangelion_1.11_You_Are_(Not)_Alone_[1080p,BluRay,x264,DTS-ES]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Eve no Jikan 2",
        "file_checksum": "88F4F7F0",
        "file_extension": "mkv",
        "file_name": "Eve no Jikan 2 [88F4F7F0].mkv"
    },
    {
        "anime_title": "EvoBot [Watakushi] Akuma no Riddle",
        "episode_number": 1,
        "file_checksum": "69A307A2",
        "file_extension": "mkv",
        "file_name": "EvoBot.[Watakushi]_Akuma_no_Riddle_-_01v2_[720p][69A307A2].mkv",
        "release_version": 2,
        "video_resolution": "720p"
    },
    {
        "anime_season": 6,
        "anime_season_prefix": "S",
        "anime_title": "Fairy Tail",
        "episode_number": 32,
        "episode_number_alt": 83,
        "episode_prefix": [
            "E",
            "Episode"
        ],
        "episode_title": "Tartaros Arc Iron Fist of the Fire Dragon",
        "file_name": "Fairy Tail - S06E32 - Tartaros Arc Iron Fist of the Fire Dragon [Episode 83]"
    },
    {
        "anime_title": "Fate Stay Night",
        "episode_number": 5,
        "episode_prefix": "Ep",
        "episode_title": "The Two Magi Part1",
        "file_extension": "mkv",
        "file_name": "Fate_Stay_Night_Ep05_The_Two_Magi_Part1_[720p,BluRay,x264]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "720p",
        "video_term": "x264"
    },
    {
        "anime_title": "Gekkan Shoujo Nozaki-kun",
        "file_name": "Gekkan Shoujo Nozaki-kun [HorribleSubs] (1080p)",
        "release_group": "HorribleSubs",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Ghost in the Shell Stand Alone Complex 2nd GIG",
        "audio_term": [
            "AAC",
            "5.1"
        ],
        "episode_number": 5,
        "episode_prefix": "Ep",
        "episode_title": "EXCAVATION",
        "file_extension": "mkv",
        "file_name": "Ghost_in_the_Shell_Stand_Alone_Complex_2nd_GIG_Ep05v2_EXCAVATION_[720p,HDTV,x264,AAC_5.1]_-_THORA.mkv",
        "release_group": "THORA",
        "release_version": 2,
        "source": "HDTV",
        "video_resolution": "720p",
        "video_term": "x264"
    },
    {
        "anime_title": "Ghost in the Shell Stand Alone Complex 2nd GIG",
        "audio_term": [
            "AAC",
            "5.1"
        ],
        "episode_number": 6,
        "episode_prefix": "Ep",
        "episode_title": "Pu239",
        "file_extension": "mkv",
        "file_name": "Ghost_in_the_Shell_Stand_Alone_Complex_2nd_GIG_Ep06_Pu239_[720p,HDTV,x264,AAC_5.1]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "HDTV",
        "video_resolution": "720p",
        "video_term": "x264"
    },
    {
        "anime_title": "Gin'iro no Kami no Agito",
        "anime_year": 2006,
        "audio_term": "DTS",
        "file_extension": "mkv",
        "file_name": "Gin'iro_no_Kami_no_Agito_(2006)_[1080p,BluRay,x264,DTS]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "Season",
        "anime_title": "Hayate no Gotoku",
        "episode_number": 24,
        "file_name": "Hayate no Gotoku 2nd Season 24 (Blu-Ray 1080p) [Chihiro]",
        "release_group": "Chihiro",
        "source": "Blu-Ray",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Howl's Moving Castle",
        "anime_year": 2004,
        "audio_term": [
            "flac",
            "dts"
        ],
        "file_extension": "mkv",
        "file_name": "Howl's_Moving_Castle_(2004)_[1080p,BluRay,flac,dts,x264]_-_THORA v2.mkv",
        "release_group": "THORA",
        "release_version": 2,
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Juuni Kokki",
        "episode_number": 5,
        "episode_prefix": "Ep.",
        "file_extension": "avi",
        "file_name": "Juuni Kokki Ep.5.avi"
    },
    {
        "anime_title": "Juuni Kokki",
        "episode_number": 5,
        "episode_prefix": "Ep.",
        "file_extension": "avi",
        "file_name": "Juuni.Kokki.Ep.5.avi"
    },
    {
        "anime_title": "Juuousei",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_checksum": "803DA487",
        "file_extension": "mkv",
        "file_name": "Juuousei_-_01_[Black_Sheep][HDTV_H264_AAC][803DA487].mkv",
        "release_group": "Black_Sheep",
        "source": "HDTV",
        "video_term": "H264"
    },
    {
        "anime_title": "K-ON!!",
        "episode_number": 8,
        "episode_prefix": "Ep",
        "episode_title": "Career Plan!",
        "file_extension": "mkv",
        "file_name": "K-ON!!_Ep08_Career_Plan!_[1080p,BluRay,x264]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "K-ON!",
        "episode_number": 3,
        "episode_prefix": "Ep",
        "episode_title": "Training!",
        "file_extension": "mkv",
        "file_name": "K-ON!_Ep03_Training!_[1080p,BluRay,x264]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Kotonoha no Niwa",
        "audio_term": "AACx3",
        "file_extension": "mp4",
        "file_name": "Kotonoha no Niwa (BD 1280x720 AVC AACx3 [5.1+2.0+2.0] Subx3).mp4",
        "other": [
            "[5.1+2.0+2.0]",
            "Subx3"
        ],
        "source": "BD",
        "video_resolution": "1280x720",
        "video_term": "AVC"
    },
    {
        "anime_title": "Macross Frontier Sayonara no Tsubasa",
        "file_checksum": "46B35E25",
        "file_extension": "mkv",
        "file_name": "Macross Frontier - Sayonara no Tsubasa (Central Anime, 720p) [46B35E25].mkv",
        "other": [
            "Central",
            "Anime"
        ],
        "video_resolution": "720p"
    },
    {
        "anime_title": "Magical Girl Lyrical Nanoha A's",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_checksum": "7A8A7769",
        "file_extension": "mkv",
        "file_name": "Magical Girl Lyrical Nanoha A's - 01.DVD[H264.AAC][DGz][7A8A7769].mkv",
        "release_group": "DGz",
        "source": "DVD",
        "video_term": "H264"
    },
    {
        "anime_title": "Mary Bell",
        "episode_number": 1,
        "file_extension": "mkv",
        "file_name": "Mary Bell (DVD) - 01v2 [h-b].mkv",
        "release_group": "h-b",
        "release_version": 2,
        "source": "DVD"
    },
    {
        "anime_title": "Mary Bell",
        "episode_number": 2,
        "file_extension": "mkv",
        "file_name": "Mary Bell (DVD) - 02 [h-b].mkv",
        "release_group": "h-b",
        "source": "DVD"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "Season",
        "anime_title": "Mobile Suit Gundam 00",
        "episode_number": 7,
        "episode_prefix": "Ep",
        "episode_title": "A Reunion and a Parting",
        "file_extension": "mkv",
        "file_name": "Mobile_Suit_Gundam_00_Season_2_Ep07_A_Reunion_and_a_Parting_[1080p,BluRay,x264]_-_THORA.mkv",
        "release_group": "THORA",
        "source": "BluRay",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Neko no Ongaeshi",
        "file_checksum": "0CDC2145",
        "file_extension": "mkv",
        "file_name": "Neko no Ongaeshi - [HQR.remux-DualAudio][NTV.1280x692.h264](0CDC2145).mkv",
        "other": "NTV",
        "release_group": "HQR.remux-DualAudio",
        "video_resolution": "1280x692",
        "video_term": "h264"
    },
    {
        "anime_title": "Noein",
        "episode_number": 1,
        "episode_total": 24,
        "file_extension": "mkv",
        "file_name": "Noein_[01_of_24]_[ru_jp]_[bodlerov_&_torrents_ru].mkv",
        "other": [
            "bodlerov",
            "&",
            "torrents",
            "ru"
        ],
        "release_group": "ru_jp"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "S",
        "anime_title": "Noragami",
        "episode_number": 6,
        "episode_prefix": [
            "E",
            "Episode"
        ],
        "episode_title": "What Must Be Done",
        "file_name": "Noragami - S02E06 - What Must Be Done [Episode 6]"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "S",
        "anime_title": "Ookiku Furikabutte",
        "episode_number": 9,
        "file_checksum": "BD841253",
        "file_extension": "mkv",
        "file_name": "Ookiku Furikabutte S2 - 09 (Central Anime) [BD841253].mkv",
        "release_group": "Central Anime"
    },
    {
        "anime_title": "Queen's Blade Utsukushiki Toushi-tachi",
        "anime_type": "OVA",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_extension": "mp4",
        "file_name": "Queen's Blade Utsukushiki Toushi-tachi - OVA_01 (BD 1280x720 AVC AAC).mp4",
        "source": "BD",
        "video_resolution": "1280x720",
        "video_term": "AVC"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "The Irregular at Magic High School",
        "episode_number": 1,
        "episode_prefix": "E",
        "episode_title": "Enrollment Part I",
        "file_extension": "mkv",
        "file_name": "The Irregular at Magic High School - S01E01- Enrollment Part I.mkv"
    },
    {
        "anime_title": "The iDOLM@STER 765 Pro to Iu Monogatari",
        "file_extension": "mkv",
        "file_name": "The iDOLM@STER 765 Pro to Iu Monogatari.mkv"
    },
    {
        "anime_title": "To Aru Kagaku no Railgun",
        "episode_number": [
            13,
            15
        ],
        "file_name": "To_Aru_Kagaku_no_Railgun_13-15_[BD_1080p][AtsA]",
        "release_group": "AtsA",
        "source": "BD",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Toradora! asdwd",
        "episode_number": 3,
        "episode_title": "Your-Song",
        "file_extension": "mkv",
        "file_name": "Toradora! asdwd-03-Your-Song.mkv"
    },
    {
        "anime_title": "Toradora",
        "episode_number": 3,
        "episode_prefix": "E",
        "episode_title": "Your-Song",
        "file_extension": "mkv",
        "file_name": "Toradora-E03-Your-Song.mkv"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Toradora!",
        "episode_number": 3,
        "episode_prefix": "E",
        "episode_title": "Your Song",
        "file_extension": "mkv",
        "file_name": "Toradora! S01E03-Your Song.mkv"
    },
    {
        "anime_title": "Tsuredure Children",
        "episode_number": 1,
        "episode_prefix": "Episódio",
        "episode_title": "Confession",
        "file_name": "Tsuredure Children Episódio 1 – Confession"
    },
    {
        "file_name": "Vol.01",
        "volume_number": 1,
        "volume_prefix": "Vol"
    },
    {
        "anime_season": 23,
        "anime_season_prefix": "S",
        "anime_title": "Nintama Rantarou",
        "episode_number": 1821,
        "episode_prefix": "E",
        "episode_title": "Buddhist Priest-sama is a Ninja",
        "file_extension": "mkv",
        "file_name": "[(´• ω •`)] Nintama Rantarou - S23E1821 - Buddhist Priest-sama is a Ninja.mkv",
        "release_group": "(´• ω •`)"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Somali and the Forest Spirit",
        "audio_term": "AAC",
        "episode_number": 1,
        "episode_prefix": "E",
        "file_checksum": "BB7C6531",
        "file_extension": "mkv",
        "file_name": "[0x539] Somali and the Forest Spirit - S01E01 (WEB 1080p Hi10P AAC) [BB7C6531].mkv",
        "other": "WEB",
        "release_group": "0x539",
        "video_resolution": "1080p",
        "video_term": "Hi10P"
    },
    {
        "anime_title": "Re:Zero kara Hajimeru Isekai Seikatsu",
        "file_name": "[46620] Re:Zero kara Hajimeru Isekai Seikatsu [1080p AV1]",
        "other": "AV1",
        "release_group": "46620",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "RWBY",
        "episode_number": 14,
        "episode_title": "Forever Fall Part 2",
        "file_extension": "mp4",
        "file_name": "[5F] RWBY - 14 Forever Fall Part 2 pt-BR.mp4",
        "language": "pt-BR",
        "release_group": "5F"
    },
    {
        "anime_title": "Neon Genesis Evangelion Platinum",
        "episode_number": 6,
        "episode_title": "Showdown in Tokyo 3",
        "file_checksum": "CBDB8577",
        "file_extension": "mkv",
        "file_name": "[ACX]Neon_Genesis_Evangelion_-_Platinum_-_06_-_Showdown_in_Tokyo_3_[SaintDeath]_[CBDB8577].mkv",
        "other": "SaintDeath",
        "release_group": "ACX"
    },
    {
        "anime_title": "Fullmetal Alchemist",
        "anime_year": 2009,
        "audio_term": "AAC",
        "episode_number": 2,
        "file_checksum": "7B2C5E8B",
        "file_extension": "mkv",
        "file_name": "[AKH-SWE]_Fullmetal_Alchemist_(2009)_02v2_[H.264.AAC][7B2C5E8B].mkv",
        "release_group": "AKH-SWE",
        "release_version": 2,
        "video_term": "H.264"
    },
    {
        "anime_title": "Canaan",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_checksum": "12F00E89",
        "file_extension": "mkv",
        "file_name": "[ANBU-Menclave]_Canaan_-_01_[1024x576_H.264_AAC][12F00E89].mkv",
        "release_group": "ANBU-Menclave",
        "video_resolution": "1024x576",
        "video_term": "H.264"
    },
    {
        "anime_title": "Haiyoru! Nyaru-Ani",
        "file_checksum": "596DD8E6",
        "file_extension": "mkv",
        "file_name": "[ANBU-umai]_Haiyoru!_Nyaru-Ani_[596DD8E6].mkv",
        "release_group": "ANBU-umai"
    },
    {
        "anime_title": "Princess Lover!",
        "episode_number": 1,
        "file_checksum": "2048A39A",
        "file_extension": "mkv",
        "file_name": "[ANBU]_Princess_Lover!_-_01_[2048A39A].mkv",
        "release_group": "ANBU"
    },
    {
        "anime_title": "Yosuga no Sora",
        "anime_type": "Preview",
        "audio_term": "FLAC",
        "episode_number": 1,
        "episode_prefix": "Ep",
        "file_extension": "mkv",
        "file_name": "[ANE] Yosuga no Sora - Ep01 Preview (Yorihime ver) [BDRip 1080p x264 FLAC].mkv",
        "other": [
            "Yorihime",
            "ver"
        ],
        "release_group": "ANE",
        "source": "BDRip",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Shingeki no Kyojin (The Final Season)",
        "audio_term": "Dual Audio",
        "episode_number": 13,
        "episode_number_alt": 72,
        "file_extension": "mkv",
        "file_name": "[AnimeRG] Shingeki no Kyojin (The Final Season) - 13 "
                     "[1080p 10bit Dual Audio] (Attack on Titan - 72).mkv",
        "other": ["Attack", "on", "Titan"],
        "release_group": "AnimeRG",
        "video_resolution": "1080p",
        "video_term": "10bit"
    },
    {
        "anime_title": "Ushio to Tora (TV)",
        "episode_number": 2,
        "file_extension": "mkv",
        "file_name": "[AnimeRG] Ushio to Tora (TV) - 02 [720p] [Xcelent].mkv",
        "other": "Xcelent",
        "release_group": "AnimeRG",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Witch Craft Works",
        "anime_type": "ED",
        "file_extension": "mkv",
        "file_name": "[AnimeRG] Witch Craft Works - EDv2 [1080p] [x265] [pseudo].mkv",
        "other": "pseudo",
        "release_group": "AnimeRG",
        "release_version": 2,
        "video_resolution": "1080p",
        "video_term": "x265"
    },
    {
        "anime_title": "Mangaka-san to Assistant-san to the Animation",
        "audio_term": "AAC",
        "episode_number": 2,
        "episode_prefix": "ep",
        "file_extension": "mp4",
        "file_name": "[AoJiaoZero][Mangaka-san to Assistant-san to the Animation] ep02 [BIG][X264_AAC][720P].mp4",
        "other": "BIG",
        "release_group": "AoJiaoZero",
        "video_resolution": "720P",
        "video_term": "X264"
    },
    {
        "anime_title": "Rozen Maiden 3",
        "anime_type": "PV",
        "file_checksum": "CA57F300",
        "file_extension": "mkv",
        "file_name": "[Asenshi] Rozen Maiden 3 - PV [CA57F300].mkv",
        "release_group": "Asenshi"
    },
    {
        "anime_title": "Infinite Stratos IS",
        "episode_number": 1,
        "file_checksum": "29675B71",
        "file_extension": "avi",
        "file_name": "[Ayako]_Infinite_Stratos_-_IS_-_01v2_[XVID][400p][29675B71].avi",
        "release_group": "Ayako",
        "release_version": 2,
        "video_resolution": "400p",
        "video_term": "XVID"
    },
    {
        "anime_title": "Kiddy Grade 2 Pilot",
        "audio_term": "AC3",
        "file_checksum": "650B731B",
        "file_extension": "mkv",
        "file_name": "[Ayu]_Kiddy_Grade_2_-_Pilot_[H264_AC3][650B731B].mkv",
        "release_group": "Ayu",
        "video_term": "H264"
    },
    {
        "anime_title": "Myself Yourself",
        "episode_number": 12,
        "file_checksum": "CB2B37F1",
        "file_extension": "mkv",
        "file_name": "[B-G_&_m.3.3.w]_Myself_Yourself_12.DVD(H.264_DD2.0)_[CB2B37F1].mkv",
        "other": "DD2.0",
        "release_group": "B-G_&_m.3.3.w",
        "source": "DVD",
        "video_term": "H.264"
    },
    {
        "anime_title": "Toradora!",
        "episode_number": 7,
        "episode_title": "Pool Opening",
        "file_checksum": "8F59F2BA",
        "file_name": "[BM&T] Toradora! - 07v2 - Pool Opening [720p Hi10 ] [BD] [8F59F2BA]",
        "release_group": "BM&T",
        "release_version": 2,
        "source": "BD",
        "video_resolution": "720p",
        "video_term": "Hi10"
    },
    {
        "anime_title": "Special A",
        "episode_number": 1,
        "file_checksum": "C83164B9",
        "file_extension": "mkv",
        "file_name": "[BakaWolf-m.3.3.w] Special A 01 (H.264) [C83164B9].mkv",
        "release_group": "BakaWolf-m.3.3.w",
        "video_term": "H.264"
    },
    {
        "anime_title": "Blue Submarine No.6",
        "audio_term": "Dual Audio",
        "file_name": "[BluDragon] Blue Submarine No.6 (DVD, R2, Dual Audio) V3",
        "other": "R2",
        "release_group": "BluDragon",
        "release_version": 3,
        "source": "DVD"
    },
    {
        "anime_title": "Sword Art Online Extra Edition",
        "audio_term": [
            "Dual Audio",
            "Vorbis"
        ],
        "file_name": "[CH] Sword Art Online Extra Edition Dual Audio [BD 480p][10bitH.264Vorbis]",
        "release_group": "CH",
        "source": "BD",
        "video_resolution": "480p",
        "video_term": [
            "10bit",
            "H.264"
        ]
    },
    {
        "anime_title": "Magical☆Star Kanon 100% OVA",
        "file_checksum": "E9F43685",
        "file_extension": "mkv",
        "file_name": "[CMS] Magical☆Star Kanon 100% OVA[DVD][E9F43685].mkv",
        "release_group": "CMS",
        "source": "DVD"
    },
    {
        "anime_title": "Kono Aozora ni Yakusoku Wo",
        "episode_number": 10,
        "file_checksum": "C83D206B",
        "file_extension": "mkv",
        "file_name": "[Chihiro]_Kono_Aozora_ni_Yakusoku_Wo_10_v2_[DVD][h264][C83D206B].mkv",
        "release_group": "Chihiro",
        "release_version": 2,
        "source": "DVD",
        "video_term": "h264"
    },
    {
        "anime_title": "Bakemonogatari",
        "anime_type": "OP",
        "audio_term": "FLAC",
        "episode_number": 4,
        "episode_part": "a",
        "file_checksum": "327A2375",
        "file_extension": "mkv",
        "file_name": "[Coalgirls]_Bakemonogatari_OP4a_(1280x720_Blu-Ray_FLAC)_[327A2375].mkv",
        "release_group": "Coalgirls",
        "source": "Blu-Ray",
        "video_resolution": "1280x720"
    },
    {
        "anime_title": "Fate Zero",
        "anime_type": "OVA",
        "audio_term": "FLAC",
        "episode_number": 3.5,
        "file_checksum": "5F5AD026",
        "file_extension": "mkv",
        "file_name": "[Coalgirls]_Fate_Zero_OVA3.5_(1280x720_Blu-ray_FLAC)_[5F5AD026].mkv",
        "release_group": "Coalgirls",
        "source": "Blu-ray",
        "video_resolution": "1280x720"
    },
    {
        "anime_title": "Toradora",
        "anime_type": "ED",
        "audio_term": "AAC",
        "episode_number": 2,
        "file_checksum": "3B65D1E6",
        "file_extension": "mkv",
        "file_name": "[Coalgirls]_Toradora_ED2_(704x480_DVD_AAC)_[3B65D1E6].mkv",
        "release_group": "Coalgirls",
        "source": "DVD",
        "video_resolution": "704x480"
    },
    {
        "anime_title": "White Album",
        "audio_term": "FLAC",
        "episode_number": [
            1,
            13
        ],
        "file_name": "[Coalgirls]_White_Album_1-13_(1280×720_Blu-Ray_FLAC)",
        "release_group": "Coalgirls",
        "source": "Blu-Ray",
        "video_resolution": "1280×720"
    },
    {
        "anime_title": "Last Exile ~Fam, The Silver Wing~",
        "episode_number": 13,
        "file_checksum": "AFF9E530",
        "file_extension": "mkv",
        "file_name": "[Commie] Last Exile ~Fam, The Silver Wing~ - 13 [AFF9E530].mkv",
        "release_group": "Commie"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "S",
        "anime_title": "Mobile Suit Gundam 00",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_checksum": "4863FBE8",
        "file_extension": "mkv",
        "file_name": "[Conclave-Mendoi]_Mobile_Suit_Gundam_00_S2_-_01v2_[1280x720_H.264_AAC][4863FBE8].mkv",
        "release_group": "Conclave-Mendoi",
        "release_version": 2,
        "video_resolution": "1280x720",
        "video_term": "H.264"
    },
    {
        "anime_title": "Bleach 225",
        "file_checksum": "C63D149C",
        "file_extension": "avi",
        "file_name": "[DB]_Bleach_225_[C63D149C].avi",
        "release_group": "DB"
    },
    {
        "anime_title": "Tatakau Shisho The Book of Bantorra",
        "audio_term": "MP3",
        "file_extension": "mkv",
        "file_name": "[Darksoul-Subs] Tatakau Shisho - The Book of Bantorra [848x480 XVID_MP3].mkv",
        "release_group": "Darksoul-Subs",
        "video_resolution": "848x480",
        "video_term": "XVID"
    },
    {
        "anime_title": "Tegami Bachi (REVERSE) Letter Bee",
        "episode_number": 4,
        "episode_number_alt": 29,
        "file_checksum": "5203142B",
        "file_extension": "mkv",
        "file_name": "[Deep] Tegami Bachi (REVERSE) - Letter Bee - 29 (04) [5203142B].mkv",
        "release_group": "Deep"
    },
    {
        "anime_title": "Magi The Labyrinth Of Magic",
        "file_name": "[DmonHiro] Magi - The Labyrinth Of Magic - Vol.1v2 (BD, 720p)",
        "release_group": "DmonHiro",
        "release_version": 2,
        "source": "BD",
        "video_resolution": "720p",
        "volume_number": 1,
        "volume_prefix": "Vol"
    },
    {
        "anime_title": "Oreshura",
        "episode_number": 1,
        "episode_prefix": "#",
        "episode_title": "The Start Of High School Life Is A War Zone",
        "file_checksum": "211375E6",
        "file_extension": "mkv",
        "file_name": "[DmonHiro] Oreshura #01v2 - The Start Of High School Life Is A War Zone [BD, 720p] [211375E6].mkv",
        "release_group": "DmonHiro",
        "release_version": 2,
        "source": "BD",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Nogizaka Haruka no Himitsu Purezza",
        "audio_term": "AAC",
        "episode_number": [
            1,
            3
        ],
        "file_name": "[Doki] Nogizaka Haruka no Himitsu - Purezza - 01v2-03v2 (1280x720 h264 AAC)",
        "release_group": "Doki",
        "release_version": [
            2,
            2
        ],
        "video_resolution": "1280x720",
        "video_term": "h264"
    },
    {
        "anime_title": "Ro-Kyu-Bu! SS 01",
        "file_checksum": "C1B5CE5D",
        "file_extension": "mkv",
        "file_name": "[Doremi].Ro-Kyu-Bu!.SS.01.[C1B5CE5D].mkv",
        "release_group": "Doremi"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Devilman Crybaby",
        "audio_term": "Multi-Audio",
        "episode_number": 4,
        "episode_prefix": "E",
        "file_checksum": "24B349D4",
        "file_extension": "mkv",
        "file_name": "[DragsterPS] Devilman Crybaby S01E04 [720p] [Multi-Audio] [Multi-Subs] [24B349D4].mkv",
        "release_group": "DragsterPS",
        "subtitles": "Multi-Subs",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Kore wa Zombie desu ka",
        "anime_type": "TV",
        "audio_term": "AAC",
        "episode_number": 3,
        "file_checksum": "888E4991",
        "file_extension": "mkv",
        "file_name": "[E-HARO Raws] Kore wa Zombie desu ka - 03 (TV 1280x720 h264 AAC) [888E4991].mkv",
        "release_group": "E-HARO Raws",
        "video_resolution": "1280x720",
        "video_term": "h264"
    },
    {
        "anime_title": "Kore wa Zombie desu ka",
        "episode_number": 2,
        "episode_prefix": "Episode",
        "file_extension": "mkv",
        "file_name": "[Edomae Subs] Kore wa Zombie desu ka  Episode 2.mkv",
        "release_group": "Edomae Subs"
    },
    {
        "anime_title": "Sora no Woto",
        "audio_term": "AAC",
        "episode_number": 7.5,
        "episode_prefix": "EP",
        "file_checksum": "C37580F8",
        "file_extension": "mkv",
        "file_name": "[Elysium]Sora.no.Woto.EP07.5(BD.720p.AAC)[C37580F8].mkv",
        "release_group": "Elysium",
        "source": "BD",
        "video_resolution": "720p"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Natsu no Arashi!",
        "episode_number": 1,
        "episode_prefix": "E",
        "episode_title": "Playback Part 2",
        "file_checksum": "BD00F464",
        "file_extension": "mkv",
        "file_name": "[EMBER] Natsu no Arashi! - S01E01-Playback Part 2 [BD00F464].mkv",
        "release_group": "EMBER"
    },
    {
        "anime_title": "One Piece",
        "episode_number": 1000,
        "file_checksum": "F2AE5FF6",
        "file_extension": "mkv",
        "file_name": "[Erai-raws] One Piece - 1000 [1080p][Multiple Subtitle][F2AE5FF6].mkv",
        "release_group": "Erai-raws",
        "subtitles": "Multiple Subtitle",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Nurse Witch Komugi-chan Magikarte",
        "episode_number": 2.5,
        "file_checksum": "902BB314",
        "file_extension": "mkv",
        "file_name": "[EroGaKi-Team]_Nurse_Witch_Komugi-chan_Magikarte_02.5_[902BB314].mkv",
        "release_group": "EroGaKi-Team"
    },
    {
        "anime_title": "AKB0048",
        "audio_term": "FLAC",
        "episode_title": "Making of Kibou-ni-Tsuite Music Video",
        "file_checksum": "C09462E2",
        "file_name": "[EveTaku] AKB0048 Vol.03 - Making of Kibou-ni-Tsuite Music Video (BDRip 1080i H.264-Hi10P FLAC)[C09462E2]",
        "release_group": "EveTaku",
        "source": "BDRip",
        "video_resolution": "1080i",
        "video_term": [
            "H.264",
            "Hi10P"
        ],
        "volume_number": 3,
        "volume_prefix": "Vol"
    },
    {
        "anime_title": "Baby Princess 3D Paradise Love",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_checksum": "457CC066",
        "file_extension": "mkv",
        "file_name": "[FBI] Baby Princess 3D Paradise Love 01v0 [BD][720p-AAC][457CC066].mkv",
        "release_group": "FBI",
        "release_version": 0,
        "source": "BD",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Crayon Shin-Chan",
        "anime_type": "Movie",
        "anime_year": 1994,
        "audio_term": "AC3",
        "episode_number": 2,
        "episode_title": "The Secret of Buri Buri Kingdom",
        "file_extension": "avi",
        "file_name": "[FB] Crayon Shin-Chan Movie 2 The Secret of Buri Buri Kingdom [DivX5 AC3] 1994 [852X480] V2.avi",
        "release_group": "FB",
        "release_version": 2,
        "video_resolution": "852X480",
        "video_term": "DivX5"
    },
    {
        "anime_title": "Futsuu no Joshikousei ga [Locodol] Yatte Mita.",
        "episode_number": 1,
        "file_checksum": "BAD09C76",
        "file_extension": "mkv",
        "file_name": "[FFF] Futsuu no Joshikousei ga [Locodol] Yatte Mita. - 01 [BAD09C76].mkv",
        "release_group": "FFF"
    },
    {
        "anime_title": "Love Live! The School Idol",
        "anime_type": ["Movie", "PV"],
        "file_checksum": "D1A15D2C",
        "file_extension": "mkv",
        "file_name": "[FFF] Love Live! The School Idol Movie - PV [D1A15D2C].mkv",
        "release_group": "FFF"
    },
    {
        "anime_title": "Red Data Girl",
        "episode_number": 10,
        "file_checksum": "29EA865B",
        "file_extension": "mkv",
        "file_name": "[FFF] Red Data Girl - 10v0 [29EA865B].mkv",
        "release_group": "FFF",
        "release_version": 0
    },
    {
        "anime_title": "Seirei Tsukai no Blade Dance",
        "anime_type": "SP",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_checksum": "F1FF8588",
        "file_extension": "mkv",
        "file_name": "[FFF] Seirei Tsukai no Blade Dance - SP01 [BD][720p-AAC][F1FF8588].mkv",
        "release_group": "FFF",
        "source": "BD",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Bakuman",
        "episode_number": 1,
        "episode_prefix": "E",
        "file_extension": "mkv",
        "file_name": "[FaggotryRaws] Bakuman - 01 (NHK E 848x480).mkv",
        "other": "NHK",
        "release_group": "FaggotryRaws",
        "video_resolution": "848x480"
    },
    {
        "anime_title": "Nodame Cantabile Finale",
        "episode_number": 0,
        "file_checksum": "73AD0735",
        "file_extension": "mkv",
        "file_name": "[Frostii]_Nodame_Cantabile_Finale_-_00_[73AD0735].mkv",
        "release_group": "Frostii"
    },
    {
        "anime_title": "Classroom Crisis",
        "audio_term": "FLAC",
        "file_name": "[GS] Classroom Crisis Vol.1&2 (BD 1080p 10bit FLAC)",
        "release_group": "GS",
        "source": "BD",
        "video_resolution": "1080p",
        "video_term": "10bit",
        "volume_number": [
            1,
            2
        ],
        "volume_prefix": "Vol."
    },
    {
        "anime_title": "Koi Kaze",
        "audio_term": "Dual Audio",
        "episode_number": 1,
        "episode_prefix": "Ep",
        "file_checksum": "c13cefe0",
        "file_extension": "mkv",
        "file_name": "[GrimRipper] Koi Kaze [Dual Audio] Ep01 (c13cefe0).mkv",
        "release_group": "GrimRipper"
    },
    {
        "file_name": "[Hakata Ramen] Cells At Work! Special.mkv",
        "file_extension": "mkv",
        "release_group": "Hakata Ramen",
        "anime_title": "Cells At Work!",
        "anime_type": "Special"
    },
    {
        "anime_title": "Evangelion 3.0 You Can (Not) Redo",
        "file_extension": "mp4",
        "file_name": "[Hakugetsu&MGRT][Evangelion 3.0 You Can (Not) Redo][480P][V0].mp4",
        "release_group": "Hakugetsu&MGRT",
        "release_version": 0,
        "video_resolution": "480P"
    },
    {
        "anime_title": "FullMetalAlchemist",
        "episode_number": 9,
        "file_extension": "rmvb",
        "file_name": "[Hard-Boiled FS]FullMetalAlchemist_09.rmvb",
        "release_group": "Hard-Boiled FS"
    },
    {
        "anime_title": "Classroom Crisis",
        "audio_term": "AAC",
        "file_name": "[Harunatsu] Classroom Crisis - Vol.1 [BD 720p-AAC]",
        "release_group": "Harunatsu",
        "source": "BD",
        "video_resolution": "720p",
        "volume_number": 1,
        "volume_prefix": "Vol."
    },
    {
        "anime_title": "Fairy Tail 2",
        "episode_number": 52,
        "episode_number_alt": 227,
        "file_checksum": "9DF6B8D5",
        "file_extension": "mkv",
        "file_name": "[Hatsuyuki-Kaitou]_Fairy_Tail_2_-_52_(227)_[720p][10bit][9DF6B8D5].mkv",
        "release_group": "Hatsuyuki-Kaitou",
        "video_resolution": "720p",
        "video_term": "10bit"
    },
    {
        "anime_title": "Dragon Ball Kai",
        "anime_year": 2014,
        "episode_number": 2,
        "episode_number_alt": 100,
        "file_checksum": "DD66AFB7",
        "file_extension": "mkv",
        "file_name": "[Hatsuyuki] Dragon Ball Kai (2014) - 002 (100) [1280x720][DD66AFB7].mkv",
        "release_group": "Hatsuyuki",
        "video_resolution": "1280x720"
    },
    {
        "anime_season": 3,
        "anime_season_prefix": "S",
        "anime_title": "Kuroko no Basuke",
        "episode_number": 1,
        "episode_number_alt": 51,
        "file_checksum": "619C57A0",
        "file_extension": "mkv",
        "file_name": "[Hatsuyuki]_Kuroko_no_Basuke_S3_-_01_(51)_[720p][10bit][619C57A0].mkv",
        "release_group": "Hatsuyuki",
        "video_resolution": "720p",
        "video_term": "10bit"
    },
    {
        "anime_title": "Kotoura-san Special Short Anime 'Haruka's Room'",
        "audio_term": "AAC",
        "episode_number": 1,
        "file_checksum": "6B6BE015",
        "file_extension": "mkv",
        "file_name": "[Hien] Kotoura-san - Special Short Anime 'Haruka's Room' - 01 [BD 1080p H.264 10-bit AAC][6B6BE015].mkv",
        "release_group": "Hien",
        "source": "BD",
        "video_resolution": "1080p",
        "video_term": [
            "H.264",
            "10-bit"
        ]
    },
    {
        "anime_title": "Sora no Woto",
        "episode_number": 1,
        "file_checksum": "E83AD672",
        "file_extension": "mkv",
        "file_name": "[Himatsubushi]_Sora_no_Woto_-_01_-_H264_-_720p_-_E83AD672.mkv",
        "release_group": "Himatsubushi",
        "video_resolution": "720p",
        "video_term": "H264"
    },
    {
        "anime_title": "Maji de Watashi ni Koi Shinasai!!",
        "episode_number": 2,
        "file_extension": "mkv",
        "file_name": "[Hiryuu] Maji de Watashi ni Koi Shinasai!! - 02 [720].mkv",
        "release_group": "Hiryuu",
        "video_resolution": "720"
    },
    {
        "anime_title": "Gintama",
        "episode_number": 111,
        "episode_part": "C",
        "file_extension": "mkv",
        "file_name": "[HorribleSubs] Gintama - 111C [1080p].mkv",
        "release_group": "HorribleSubs",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "HEROMAN",
        "episode_number": 10,
        "file_extension": "mkv",
        "file_name": "[HorribleSubs] HEROMAN - 10_(XviD_AnimeSenshi).mkv",
        "other": "AnimeSenshi",
        "release_group": "HorribleSubs",
        "video_term": "XviD"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "S",
        "anime_title": "Mob Psycho 100",
        "episode_number": 7,
        "file_extension": "mkv",
        "file_name": "[HorribleSubs] Mob Psycho 100 S2 - 07 [1080p].mkv",
        "release_group": "HorribleSubs",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Momokuri",
        "episode_number": [
            1,
            2
        ],
        "file_name": "[HorribleSubs] Momokuri - 01+02 [720p]",
        "release_group": "HorribleSubs",
        "video_resolution": "720p"
    },
    {
        "anime_title": "The World God Only Knows 2",
        "episode_number": 3,
        "file_extension": "mkv",
        "file_name": "[HorribleSubs] The World God Only Knows 2 - 03 [720p].mkv",
        "release_group": "HorribleSubs",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Tower of Druaga Sword of Uruk",
        "episode_number": 4,
        "file_extension": "mkv",
        "file_name": "[HorribleSubs] Tower of Druaga - Sword of Uruk - 04 [480p].mkv",
        "release_group": "HorribleSubs",
        "video_resolution": "480p"
    },
    {
        "anime_title": "Tsukimonogatari",
        "episode_number": [
            1,
            4
        ],
        "file_extension": "mkv",
        "file_name": "[HorribleSubs] Tsukimonogatari - (01-04) [1080p].mkv",
        "release_group": "HorribleSubs",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Norn9 Norn + Nonetto",
        "episode_number": 12,
        "file_name": "[Infantjedi] Norn9 - Norn + Nonetto - 12",
        "release_group": "Infantjedi"
    },
    {
        "anime_title": "Heavy Object",
        "anime_type": "NCED",
        "episode_number": 1,
        "file_extension": "mkv",
        "file_name": "[Judas[ Heavy Object - NCED 01.mkv",
        "release_group": "Judas"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Aharen-san wa Hakarenai",
        "episode_number": 6,
        "episode_prefix": "E",
        "file_extension": "mkv",
        "file_name": "[Judas] Aharen-san wa Hakarenai - S01E06v2.mkv",
        "release_group": "Judas",
        "release_version": 2
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Amanchu!",
        "anime_type": "Ending",
        "file_extension": "mkv",
        "file_name": "[Judas] Amanchu! S1 - Ending.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Black Clover Jump Special Anime Festa 2018",
        "file_name": "[Judas] Black Clover - Jump Special Anime Festa 2018.mkv",
        "file_extension": "mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Black Clover",
        "anime_type": "NCOP",
        "episode_number": 10,
        "episode_part": "b",
        "episode_title": "Black Catcher",
        "file_name": "[Judas] Black Clover - NCOP 10b - Black Catcher.mkv",
        "file_extension": "mkv",
        "release_group": "Judas",
    },
    {
        "anime_title": "Boruto",
        "anime_type": "OVA",
        "episode_title": "The Day Naruto Became Hokage",
        "file_name": "[Judas] Boruto - OVA - The Day Naruto Became Hokage.mkv",
        "file_extension": "mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Card Captor Sakura The Movie",
        "episode_number": 2,
        "episode_title": "The Sealed Card",
        "file_extension": "mkv",
        "file_name": "[Judas] Card Captor Sakura - The Movie 2 - The Sealed Card.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Code Geass",
        "anime_type": "Movie",
        "episode_number": 2,
        "file_extension": "mkv",
        "file_name": "[Judas] Code Geass Movie 02.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Code Geass",
        "anime_type": "Movie",
        "episode_number": 2,
        "episode_title": "Lelouch of the Rebellion Transgression",
        "file_extension": "mkv",
        "file_name": "[Judas] Code Geass Movie 02 - Lelouch of the Rebellion - Transgression.mkv",
        "release_group": "Judas"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Gintama",
        "episode_number": [1, 2],
        "episode_number_alt": [1, 2],
        "episode_prefix": ["E", "E"],
        "file_extension": "mkv",
        "file_name": "[Judas] Gintama - S01E01-E02 (001-002).mkv",
        "release_group": "Judas"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "S",
        "anime_title": "Gintama",
        "episode_number": [1, 2],
        "episode_number_alt": [25, 26],
        "episode_prefix": ["E", "E"],
        "file_extension": "mkv",
        "file_name": "[Judas] Gintama - S02E01-E02 (025-026).mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Granblue Fantasy",
        "anime_type": "Clean Ending",
        "file_extension": "mkv",
        "file_name": "[Judas] Granblue Fantasy - Clean Ending.mkv",
        "release_group": "Judas"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "Grisaia no Kajitsu",
        "anime_type": "S",
        "episode_number": 2,
        "file_extension": "mkv",
        "file_name": "[Judas] Grisaia no Kajitsu - S01S02.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Initial D Extra Stage",
        "episode_number": 2,
        "episode_title": "Sentimental White",
        "file_extension": "mkv",
        "file_name": "[Judas] Initial D - Extra Stage 02 - Sentimental White.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Initial D Extra Stage 2 Sentimental White",
        "file_extension": "mkv",
        "file_name": "[Judas] Initial D - Extra Stage 2 - Sentimental White.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "Nogizaka Haruka no Himitsu",
        "anime_type": ["OVA", "NCOP"],
        "file_extension": "mkv",
        "file_name": "[Judas] Nogizaka Haruka no Himitsu - OVA NCOP.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "One Piece",
        "episode_number": 1009,
        "file_extension": "mkv",
        "file_name": "[Judas] One Piece - 1009v2.mkv",
        "release_group": "Judas",
        "release_version": 2
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "S",
        "anime_title": "One Punch Man",
        "anime_type": "OVA",
        "episode_number": 2,
        "file_extension": "mkv",
        "file_name": "[Judas] One Punch Man - S01OVA02.mkv",
        "release_group": "Judas"
    },
    {
        "anime_season": 1,
        "anime_season_prefix": "Season",
        "anime_title": "One-Punch Man",
        "anime_type": "OVA",
        "episode_number": 5,
        "file_extension": "mkv",
        "file_name": "[Judas] One-Punch Man Season 1 OVA - 05.mkv",
        "release_group": "Judas"
    },
    {
        "anime_title": "D.C.II Da Capo II",
        "episode_number": 1,
        "episode_prefix": "Ep",
        "file_checksum": "a1fc58a7",
        "file_extension": "mkv",
        "file_name": "[Jumonji-Giri]_[Shinsen-Subs][ASF]_D.C.II_Da_Capo_II_Ep01_(a1fc58a7).mkv",
        "other": [
            "Shinsen-Subs",
            "ASF"
        ],
        "release_group": "Jumonji-Giri"
    },
    {
        "anime_title": "One Piece",
        "anime_type": "Movie",
        "episode_number": 9,
        "file_extension": "avi",
        "file_name": "[KAF-TEAM]_One_Piece_Movie_9_vostfr_HD.avi",
        "language": "vostfr",
        "release_group": "KAF-TEAM",
        "video_term": "HD"
    },
    {
        "anime_title": "D.Gray-man",
        "episode_number": 4,
        "file_extension": "avi",
        "file_name": "[KLF]_D.Gray-man_04V2.avi",
        "release_group": "KLF",
        "release_version": 2
    },
    {
        "anime_title": "Uchuu no Stellvia",
        "audio_term": "AAC",
        "episode_number": 14,
        "episode_prefix": "ep",
        "file_checksum": "06EE7355",
        "file_extension": "mkv",
        "file_name": "[Kira-Fansub] Uchuu no Stellvia ep 14 (BD H264 1280x960 24fps AAC) [06EE7355].mkv",
        "release_group": "Kira-Fansub",
        "source": "BD",
        "video_resolution": "1280x960",
        "video_term": [
            "H264",
            "24fps"
        ]
    },
    {
        "anime_title": "1001 Nights",
        "anime_year": 1998,
        "file_name": "[LRL] 1001 Nights (1998) [DVD]",
        "release_group": "LRL",
        "source": "DVD"
    },
    {
        "anime_title": "Umineko no Naku Koro ni",
        "audio_term": "AAC",
        "episode_number": 11,
        "file_checksum": "943106AD",
        "file_extension": "mkv",
        "file_name": "[Lambda-Delta]_Umineko_no_Naku_Koro_ni_-_11_[848x480_H.264_AAC][943106AD].mkv",
        "release_group": "Lambda-Delta",
        "video_resolution": "848x480",
        "video_term": "H.264"
    },
    {
        "anime_title": "Aikatsu!",
        "episode_number": 100,
        "file_checksum": "D035A39F",
        "file_extension": "mkv",
        "file_name": "[Mezashite] Aikatsu! ‒ 100 [D035A39F].mkv",
        "release_group": "Mezashite"
    },
    {
        "anime_title": "Angel Beats",
        "episode_number": 9,
        "file_extension": "mkv",
        "file_name": "[N LogN Fansubs] Angel Beats (9).mkv",
        "release_group": "N LogN Fansubs"
    },
    {
        "anime_title": "Hidamari Sketch x365",
        "episode_number": 9,
        "episode_part": "a",
        "file_checksum": "49874745",
        "file_extension": "mkv",
        "file_name": "[NamaeNai] Hidamari Sketch x365 - 09a (DVD) [49874745].mkv",
        "release_group": "NamaeNai",
        "source": "DVD"
    },
    {
        "anime_title": "Tiger & Bunny",
        "audio_term": "AAC",
        "episode_number": 1,
        "episode_prefix": "#",
        "episode_title": "All's well that ends well.",
        "file_checksum": "4A9AB85F",
        "file_extension": "mkv",
        "file_name": "[NinjaPanda] Tiger & Bunny #01 All's well that ends well. (v3, 1080p Hi10P, DA AAC) [4A9AB85F].mkv",
        "other": "DA",
        "release_group": "NinjaPanda",
        "release_version": 3,
        "video_resolution": "1080p",
        "video_term": "Hi10P"
    },
    {
        "anime_title": "Tamayura ~graduation photo~ Movie Part 1",
        "file_checksum": "98965607",
        "file_extension": "mkv",
        "file_name": "[Nishi-Taku] Tamayura ~graduation photo~ Movie Part 1 [BD][720p][98965607].mkv",
        "release_group": "Nishi-Taku",
        "source": "BD",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Space Battleship Yamato 2199",
        "anime_year": 2012,
        "audio_term": "AAC",
        "episode_number": 18,
        "episode_prefix": "episode",
        "file_checksum": "1F56D642",
        "file_name": "[Nubles] Space Battleship Yamato 2199 (2012) episode 18 (720p 10 bit AAC)[1F56D642]",
        "release_group": "Nubles",
        "video_resolution": "720p",
        "video_term": "10 bit"
    },
    {
        "anime_title": "Space Battleship Yamato 2199",
        "anime_year": 2012,
        "audio_term": "AAC",
        "episode_number": 18,
        "episode_prefix": "episode",
        "file_checksum": "BA70BA9C",
        "file_name": "[Nubles] Space Battleship Yamato 2199 (2012) episode 18 (720p 8 bit AAC)[BA70BA9C]",
        "release_group": "Nubles",
        "video_resolution": "720p",
        "video_term": "8 bit"
    },
    {
        "anime_title": "Diebuster",
        "audio_term": "AC3",
        "episode_number": 1,
        "episode_prefix": "EP",
        "file_checksum": "82E36A36",
        "file_extension": "mkv",
        "file_name": "[R-R] Diebuster.EP1 (720p.Hi10p.AC3) [82E36A36].mkv",
        "release_group": "R-R",
        "video_resolution": "720p",
        "video_term": "Hi10p"
    },
    {
        "anime_title": "Sakura Taisen New York NY",
        "episode_number": 2,
        "episode_prefix": "Ep",
        "file_checksum": "1590D378",
        "file_extension": "avi",
        "file_name": "[RNA]_Sakura_Taisen_New_York_NY_Ep_2_[1590D378].avi",
        "release_group": "RNA"
    },
    {
        "anime_title": "Mezzo(DSA)",
        "audio_term": "ogg",
        "episode_number": 5,
        "file_checksum": "585d9971",
        "file_extension": "mkv",
        "file_name": "[RaX]Mezzo(DSA)_-_05_-_[x264_ogg]_[585d9971].mkv",
        "release_group": "RaX",
        "video_term": "x264"
    },
    {
        "anime_title": "Persona 4 The Animation",
        "audio_term": "FLAC",
        "episode_number": 13,
        "episode_prefix": "Episode",
        "episode_title": "A Stormy Summer Vacation Part 1",
        "file_checksum": "8A45634B",
        "file_extension": "mkv",
        "file_name": "[Raizel] Persona 4 The Animation Episode 13 - A Stormy Summer Vacation Part 1  [BD_1080p_Dual_Audio_FLAC_Hi10p][8A45634B].mkv",
        "other": [
            "Dual",
            "Audio"
        ],
        "release_group": "Raizel",
        "source": "BD",
        "video_resolution": "1080p",
        "video_term": "Hi10p"
    },
    {
        "anime_title": "Gift ~eternal rainbow~",
        "audio_term": "vorbis",
        "episode_number": 1,
        "file_extension": "mkv",
        "file_name": "[Rakuda].Gift.~eternal.rainbow~.01.dvd.h.264.vorbis.mkv",
        "release_group": "Rakuda",
        "source": "dvd",
        "video_term": "h.264"
    },
    {
        "anime_title": "Memories Off 3.5",
        "episode_number": 4,
        "file_extension": "mkv",
        "file_name": "[ReDone] Memories Off 3.5 - 04 (DVD 10-bit).mkv",
        "release_group": "ReDone",
        "source": "DVD",
        "video_term": "10-bit"
    },
    {
        "anime_title": "Serial Experiments Lain",
        "audio_term": "Dual-Audio",
        "episode_number": 1,
        "episode_prefix": "E",
        "file_extension": "mkv",
        "file_name": "[Reaktor] Serial Experiments Lain - E01 [1080p][x265][10-bit][Dual-Audio].mkv",
        "release_group": "Reaktor",
        "video_resolution": "1080p",
        "video_term": [
            "x265",
            "10-bit"
        ]
    },
    {
        "anime_title": "No.6",
        "episode_number": 1,
        "file_checksum": "A956075C",
        "file_extension": "mkv",
        "file_name": "[Ruri]No.6 01 [720p][H264][A956075C].mkv",
        "release_group": "Ruri",
        "video_resolution": "720p",
        "video_term": "H264"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "S",
        "anime_title": "Queen's Blade",
        "file_name": "[SFW]_Queen's_Blade_S2",
        "release_group": "SFW"
    },
    {
        "anime_title": "Kemono no Souja Erin",
        "episode_number": 12,
        "file_checksum": "0F5F884F",
        "file_extension": "mkv",
        "file_name": "[SS]_Kemono_no_Souja_Erin_-_12_(1280x720_h264)_[0F5F884F].mkv",
        "release_group": "SS",
        "video_resolution": "1280x720",
        "video_term": "h264"
    },
    {
        "anime_title": "AIKa ZERO",
        "anime_type": "OVA",
        "audio_term": "Flac",
        "episode_number": 1,
        "file_checksum": "6730D40A",
        "file_extension": "mkv",
        "file_name": "[Seto_Otaku]_AIKa_ZERO_OVA_-_01_[BD][1920x1080_H264-Flac][6730D40A].mkv",
        "release_group": "Seto_Otaku",
        "source": "BD",
        "video_resolution": "1920x1080",
        "video_term": "H264"
    },
    {
        "anime_title": "Macross Frontier",
        "episode_number": 1,
        "episode_part": "b",
        "file_checksum": "4D5EC315",
        "file_extension": "avi",
        "file_name": "[Shinsen-Subs]_Macross_Frontier_-_01b_[4D5EC315].avi",
        "release_group": "Shinsen-Subs"
    },
    {
        "anime_title": "Hidamari Sketch x365",
        "episode_number": 4.1,
        "file_checksum": "B6CE8458",
        "file_extension": "mkv",
        "file_name": "[SpoonSubs]_Hidamari_Sketch_x365_-_04.1_(DVD)[B6CE8458].mkv",
        "release_group": "SpoonSubs",
        "source": "DVD"
    },
    {
        "anime_title": "Swing out Sisters",
        "audio_term": "AC3",
        "file_checksum": "3ABD57E6",
        "file_extension": "mp4",
        "file_name": "[SubDESU-H] Swing out Sisters Complete Version (720p x264 8bit AC3) [3ABD57E6].mp4",
        "other": "Version",
        "release_group": "SubDESU-H",
        "release_information": "Complete",
        "video_resolution": "720p",
        "video_term": [
            "x264",
            "8bit"
        ]
    },
    {
        "anime_title": "Kidou Senshi Gundam UC Unicorn",
        "episode_number": 2,
        "episode_prefix": "episode",
        "file_extension": "mp4",
        "file_name": "[TV-J] Kidou Senshi Gundam UC Unicorn - episode.02 [BD 1920x1080 h264+AAC(5.1ch JP+EN) +Sub(JP-EN-SP-FR-CH) Chap].mp4",
        "other": [
            "AAC(5",
            "1ch",
            "JP",
            "EN)",
            "Sub(JP-EN-SP-FR-CH)",
            "Chap"
        ],
        "release_group": "TV-J",
        "source": "BD",
        "video_resolution": "1920x1080",
        "video_term": "h264"
    },
    {
        "anime_title": "Toradora!",
        "anime_year": 2008,
        "audio_term": "FLAC",
        "episode_number": 1,
        "episode_title": "Tiger and Dragon",
        "file_checksum": "1234ABCD",
        "file_extension": "mkv",
        "file_name": "[TaigaSubs]_Toradora!_(2008)_-_01v2_-_Tiger_and_Dragon_[1280x720_H.264_FLAC][1234ABCD].mkv",
        "release_group": "TaigaSubs",
        "release_version": 2,
        "video_resolution": "1280x720",
        "video_term": "H.264"
    },
    {
        "anime_title": "Fullmetal Alchemist",
        "anime_year": 2009,
        "episode_number": 4,
        "file_checksum": "40F2A957",
        "file_extension": "mp4",
        "file_name": "[Taka]_Fullmetal_Alchemist_(2009)_04_[720p][40F2A957].mp4",
        "release_group": "Taka",
        "video_resolution": "720p"
    },
    {
        "anime_title": "Today in Class 5-2",
        "episode_number": 4,
        "file_extension": "avi",
        "file_name": "[Triad]_Today_in_Class_5-2_-_04.avi",
        "release_group": "Triad"
    },
    {
        "anime_title": "Hyouka",
        "audio_term": "FLAC",
        "episode_number": [
            1,
            4
        ],
        "file_name": "[Tsundere] Hyouka - 01v2-04 [BDRip h264 1920x1080 10bit FLAC]",
        "release_group": "Tsundere",
        "release_version": 2,
        "source": "BDRip",
        "video_resolution": "1920x1080",
        "video_term": [
            "h264",
            "10bit"
        ]
    },
    {
        "anime_title": "Evangelion 3.33 You Can (Not) Redo",
        "audio_term": "flac",
        "file_checksum": "F2060CF5",
        "file_extension": "mkv",
        "file_name": "[UTW-THORA] Evangelion 3.33 You Can (Not) Redo [BD][1080p,x264,flac][F2060CF5].mkv",
        "release_group": "UTW-THORA",
        "source": "BD",
        "video_resolution": "1080p",
        "video_term": "x264"
    },
    {
        "anime_title": "Summer Wars",
        "audio_term": "TrueHD5.1",
        "file_checksum": "9F311DAB",
        "file_extension": "mkv",
        "file_name": "[UTW-TMD]_Summer_Wars_[BD][h264-720p][TrueHD5.1][9F311DAB].mkv",
        "release_group": "UTW-TMD",
        "source": "BD",
        "video_resolution": "720p",
        "video_term": "h264"
    },
    {
        "anime_title": "Accel World EX01",
        "audio_term": "AAC",
        "file_checksum": "3E56EE18",
        "file_extension": "mkv",
        "file_name": "[UTW]_Accel_World_-_EX01_[BD][h264-720p_AAC][3E56EE18].mkv",
        "release_group": "UTW",
        "source": "BD",
        "video_resolution": "720p",
        "video_term": "h264"
    },
    {
        "anime_title": "Fate Zero",
        "audio_term": "AC3",
        "episode_number": 1,
        "file_checksum": "02A0491D",
        "file_extension": "mkv",
        "file_name": "[UTW]_Fate_Zero_-_01_[BD][h264-720p_AC3][02A0491D].mkv",
        "release_group": "UTW",
        "source": "BD",
        "video_resolution": "720p",
        "video_term": "h264"
    },
    {
        "anime_title": "Bokura Ga Ita",
        "audio_term": "AC3",
        "episode_number": 1,
        "file_checksum": "BFCE1627",
        "file_extension": "mkv",
        "file_name": "[Urusai]_Bokura_Ga_Ita_01_[DVD_h264_AC3]_[BFCE1627][Fixed].mkv",
        "other": "Fixed",
        "release_group": "Urusai",
        "source": "DVD",
        "video_term": "h264"
    },
    {
        "anime_title": "First Squad The Morment Of Truth",
        "file_extension": "mkv",
        "file_name": "[ValdikSS]_First_Squad_The_Morment_Of_Truth_[720x576_h264_dvdscr_eng_hardsub].mkv",
        "language": "eng",
        "other": "dvdscr",
        "release_group": "ValdikSS",
        "subtitles": "hardsub",
        "video_resolution": "720x576",
        "video_term": "h264"
    },
    {
        "anime_title": "009-1",
        "episode_number": 2,
        "file_checksum": "36D2444D",
        "file_extension": "mkv",
        "file_name": "[Yoroshiku]_009-1_-_02_(H264)_[36D2444D].mkv",
        "release_group": "Yoroshiku",
        "video_term": "H264"
    },
    {
        "anime_title": "Darker than Black Gemini of the Meteor",
        "episode_number": 1,
        "file_checksum": "65274FDE",
        "file_extension": "7z",
        "file_name": "[Yuurisan-Subs]_Darker_than_Black_-_Gemini_of_the_Meteor_-_01v2_[65274FDE].patch.7z",
        "release_group": "Yuurisan-Subs",
        "release_information": "patch",
        "release_version": 2
    },
    {
        "anime_title": "Shingeki no Kyojin",
        "audio_term": "AAC",
        "episode_number": 25,
        "file_extension": "mp4",
        "file_name": "[Zero-Raws] Shingeki no Kyojin - 25 END (MBS 1280x720 x264 AAC).mp4",
        "other": "MBS",
        "release_group": "Zero-Raws",
        "release_information": "END",
        "video_resolution": "1280x720",
        "video_term": "x264"
    },
    {
        "anime_title": "Machine-Doll wa Kizutsukanai",
        "episode_number": 1,
        "episode_title": "Facing ''Cannibal Candy'' I",
        "file_checksum": "B99C8DED",
        "file_extension": "mkv",
        "file_name": "[Zom-B] Machine-Doll wa Kizutsukanai - 01 - Facing ''Cannibal Candy'' I (kuroi, FFF remux) [B99C8DED].mkv",
        "other": [
            "kuroi",
            "FFF"
        ],
        "release_group": "Zom-B",
        "release_information": "remux"
    },
    {
        "anime_title": "Sora no Woto",
        "audio_term": "AAC",
        "episode_number": 7.5,
        "episode_title": "Drinking Party Fortress Battle",
        "file_checksum": "F7DF16F7",
        "file_extension": "mkv",
        "file_name": "[Zurako] Sora no Woto - 07.5 - Drinking Party - Fortress Battle (BD 1080p AAC) [F7DF16F7].mkv",
        "release_group": "Zurako",
        "source": "BD",
        "video_resolution": "1080p"
    },
    {
        "anime_title": "Nanatsu no Taizai S3 Kamigami no Gekirin",
        "episode_number": 21,
        "file_extension": "mkv",
        "file_name": "[zza] Nanatsu no Taizai - S03 - Kamigami no Gekirin - 21 [1080p.x265].mkv",
        "release_group": "zza",
        "video_resolution": "1080p",
        "video_term": "x265"
    },
    {
        "anime_season": 3,
        "anime_season_prefix": "S",
        "anime_title": "Nanatsu no Taizai",
        "anime_type": "OVA",
        "episode_number": 21,
        "file_extension": "mkv",
        "file_name": "[zza] Nanatsu no Taizai - S03 - OVA - 21 [1080p.x265].mkv",
        "release_group": "zza",
        "video_resolution": "1080p",
        "video_term": "x265"
    },
    {
        "anime_title": "Shingeki no Kyojin",
        "audio_term": "AAC",
        "episode_number": 5,
        "file_extension": "mp4",
        "file_name": "[[Zero-Raws] Shingeki no Kyojin - 05 (MBS 1280x720 x264 AAC).mp4",
        "other": "MBS",
        "release_group": "Zero-Raws",
        "video_resolution": "1280x720",
        "video_term": "x264"
    },
    {
        "anime_title": "R.O.D the TV",
        "episode_number": 1,
        "file_extension": "mkv",
        "file_name": "[a4e]R.O.D_the_TV_01[divx5.2.1].mkv",
        "other": "divx5.2.1",
        "release_group": "a4e"
    },
    {
        "anime_title": "Seikon no Qwaser",
        "episode_number": 13,
        "file_checksum": "988DB090",
        "file_extension": "mkv",
        "file_name": "[chibi-Doki] Seikon no Qwaser - 13v0 (Uncensored Director's Cut) [988DB090].mkv",
        "other": [
            "Uncensored",
            "Director's",
            "Cut"
        ],
        "release_group": "chibi-Doki",
        "release_version": 0
    },
    {
        "anime_title": "Lupin III Kutabare! Nostradamus",
        "audio_term": "MultiAudio",
        "file_extension": "mkv",
        "file_name": "[fong] Lupin III - Kutabare! Nostradamus [BDrip.1080p.10bit.MultiAudio].mkv",
        "release_group": "fong",
        "source": "BDrip",
        "video_resolution": "1080p",
        "video_term": "10bit"
    },
    {
        "anime_season": 2,
        "anime_season_prefix": "Season",
        "anime_title": "Kimi ni Todoke",
        "episode_number": 0,
        "file_checksum": "BF735BC4",
        "file_extension": "mkv",
        "file_name": "[gg]_Kimi_ni_Todoke_2nd_Season_-_00_[BF735BC4].mkv",
        "release_group": "gg"
    },
    {
        "anime_title": "Nazca",
        "episode_number": 1,
        "episode_prefix": "episode",
        "file_extension": "mkv",
        "file_name": "[kito].Nazca.episode.01.DVDRip.[x264.He-aac.{Jpn}+Sub{Fr}].mkv",
        "other": [
            "He-aac",
            "{Jpn}",
            "Sub{Fr}"
        ],
        "release_group": "kito",
        "source": "DVDRip",
        "video_term": "x264"
    },
    {
        "anime_title": "Natsume Yuujinchou Shi",
        "audio_term": "AAC",
        "file_name": "[tlacatlc6] Natsume Yuujinchou Shi Vol. 1v2 & Vol. 2 (BD 1280x720 x264 AAC)",
        "release_group": "tlacatlc6",
        "release_version": 2,
        "source": "BD",
        "video_resolution": "1280x720",
        "video_term": "x264",
        "volume_number": [
            1,
            2
        ],
        "volume_prefix": [
            "Vol.",
            "Vol."
        ]
    },
    {
        "anime_title": "Elfen Lied",
        "audio_term": [
            "5.1",
            "AAC"
        ],
        "episode_number": 12,
        "file_extension": "mkv",
        "file_name": "]AkihitoSubs] Elfen Lied - 12 [BD 1920x1080 x265 10Bit 5.1 AAC].mkv",
        "release_group": "AkihitoSubs",
        "source": "BD",
        "video_resolution": "1920x1080",
        "video_term": [
            "x265",
            "10Bit"
        ]
    }
]



import re
from typing import List

from aniparse import parser_helper
from aniparse.element import ElementCategory
from aniparse.keyword import Keyword
from aniparse.token import Token, TokenCategory, TokenFlags, Tokens

ANIME_YEAR_MIN = 1900
ANIME_YEAR_MAX = 2100


# noinspection DuplicatedCode
class EpisodePattern(Tokens):
    _re_japanese_counter = re.compile(r"^(第)?(\d+)"  # prefix
                                      r"(?:([" + parser_helper.ESCAPED_DASHES +
                                      r" ~&+]?)"  # prefix separator
                                      r"(\d+))?"  # number
                                      r"([話弾])$",  # prefix
                                      flags=re.IGNORECASE)
    _re_number_sign = re.compile(r"^(#)(\d+(?:\.\d+)?)"  # sign and number
                                 r"(?:([" + parser_helper.ESCAPED_DASHES +
                                 r" ~&+]?)"  # separator
                                 r"(\d+(?:\.\d+)?))?"  # 2nd number
                                 r"(?:(v)(\d+))?$",  # version
                                 flags=re.IGNORECASE)
    _re_season_episode = re.compile(r"^(?:(\d+)|(s)(\d+)"  # Season prefix or number
                                    r"(?:([" + parser_helper.ESCAPED_DASHES +
                                    r" ~&+])"  # separator
                                    r"(s)?(\d+))?)"  # 2nd season prefix or number
                                    # Season "1", "s01", "s01-02", "s01~02"
                                    r"(?:(x)|([" + parser_helper.ESCAPED_DASHES +
                                    r" ._\-x:])?(e))"  # Season-Episode separator
                                    r"(\d+(?:\.\d+)?)(?:(v)(\d+))?"  # Episode "01", "01v2"
                                    r"(?:([" + parser_helper.ESCAPED_DASHES +
                                    r"~&+])(e)?"  # Episode range separator
                                    r"(\d+(?:\.\d+)?)(?:(v)(\d+))?)?$",  # Episode "01", "01v2"
                                    flags=re.IGNORECASE)

    def is_match_japanese_counter_pattern(self, token: Token, word: str) -> bool:
        """
        Checking for japanese style counter pattern

        e.g. "01話", "第01話", "第02-03話"
        pattern
        <japanese counter><episode number>[<range_separator><episode number>]
        """
        match = self._re_japanese_counter.match(word)
        if match:
            if match.group(5):
                self.insert_tokens(token, match.group(5), TokenCategory.IDENTIFIER, ElementCategory.UNKNOWN)
            if match.group(4):
                self.insert_tokens(token, match.group(4), TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
            if match.group(3):
                self.insert_tokens(token, match.group(3), TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
            if match.group(2):
                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                token.content = match.group(2)
            if match.group(1):
                new_token = Token(match.group(1), TokenCategory.IDENTIFIER, ElementCategory.EPISODE_PREFIX,
                                  token.enclosed)
                self.insert_before(token, new_token)
            return True
        return False

    def is_match_number_sign_pattern(self, token: Token, word: str) -> bool:
        """
        Checking for number sign pattern
        e.g. "#01", "#02-03v2"
        pattern
        <number sign><episode number>[<range_separator><episode number>][<version>]
        """
        match = self._re_number_sign.match(word)
        if match:
            if match.group(6):
                self.insert_tokens(token, match.group(6), TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION)
            if match.group(5):
                self.insert_tokens(token, match.group(5), TokenCategory.INVALID, ElementCategory.UNKNOWN)
            if match.group(4):
                self.insert_tokens(token, match.group(4), TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
            if match.group(3):
                self.insert_tokens(token, match.group(3), TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
            if match.group(2):
                self.insert_tokens(token, match.group(2), TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
            self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_PREFIX)

            token.content = match.group(1)
            return True
        return False

    def is_match_season_and_episode_pattern(self, token: Token, word: str) -> bool:
        """
        Checking for season and episode pattern
        e.g "2x01", "2x01v2"
        e.g "s01e03", "s01e03v2"
        e.g "s01-02xe001-150", "s01-02xe001-150v2", "s01-02xe001v3-150v2"
        e.g "s01-s02xe001-150", "s01-s02xe001-150v2", "s01-s02xe001v3-150v2"
        e.g "s01e03-s02e04", "s01e03-s02e04v2", "s01e03v4-s02e04v2"
        e.g "s01e03-04", "s01e03-04v2", "s01e03v4-04v2"
        assuming, previous was season: "01-02e03", "01-02e03v2", "01v3-s02e03v2"
        pattern:
        (<season_number>|<season_prefix><season_number>[<range_separator>[season_prefix]<season_number>])
        (<range_separator>|[range_separator]<episode_prefix>)
        <episode_number>[v<release_version>][<range_separator>[episode_prefix]<episode_number>[v<release_version>]]
        """

        match = self._re_season_episode.match(word)

        if match:
            # check for lower bound < upper bound
            season_lower_bound = match.group(1) or match.group(3)
            season_upper_bound = match.group(6)
            episode_lower_bound = match.group(10)
            episode_upper_bound = match.group(15)

            if season_upper_bound and int(season_lower_bound) > int(season_upper_bound):
                return False
            if episode_upper_bound and int(episode_lower_bound) > int(episode_upper_bound):
                return False
            try:
                if int(season_lower_bound) == 0 and token.enclosed:
                    # Avoid fansubbers [0x539] as season pattern
                    return False
            except ValueError:
                pass
            # in reverse order to keep the order of tokens
            if match.group(17):
                self.insert_tokens(token, match.group(17), TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION)
            if match.group(16):
                self.insert_tokens(token, match.group(16), TokenCategory.INVALID, ElementCategory.UNKNOWN)
            if match.group(15):
                self.insert_tokens(token, episode_upper_bound, TokenCategory.IDENTIFIER,
                                   ElementCategory.EPISODE_NUMBER)
            if match.group(14):
                self.insert_tokens(token, match.group(14), TokenCategory.IDENTIFIER, ElementCategory.EPISODE_PREFIX)
            if match.group(13):
                self.insert_tokens(token, match.group(13), TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
            if match.group(12):
                self.insert_tokens(token, match.group(12), TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION)
            if match.group(11):
                self.insert_tokens(token, match.group(11), TokenCategory.INVALID, ElementCategory.UNKNOWN)
            if match.group(10):
                self.insert_tokens(token, match.group(10), TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
            if match.group(9):
                self.insert_tokens(token, match.group(9), TokenCategory.IDENTIFIER, ElementCategory.EPISODE_PREFIX)
            if match.group(8):
                self.insert_tokens(token, match.group(8), TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
            if match.group(7):
                self.insert_tokens(token, match.group(7), TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
            if match.group(6):
                self.insert_tokens(token, match.group(6), TokenCategory.IDENTIFIER, ElementCategory.ANIME_SEASON)
            if match.group(5):
                self.insert_tokens(token, match.group(5), TokenCategory.IDENTIFIER,
                                   ElementCategory.ANIME_SEASON_PREFIX)
            if match.group(4):
                self.insert_tokens(token, match.group(4), TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
            if match.group(2):
                if match.group(3):
                    self.insert_tokens(token, match.group(3), TokenCategory.IDENTIFIER, ElementCategory.ANIME_SEASON)
                token.content = match.group(2)
                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_SEASON_PREFIX)
            elif match.group(1):
                token.content = match.group(1)
                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_SEASON)
            return True
        return False


class VolumePattern(Tokens):
    pass


class Pattern(EpisodePattern, VolumePattern):
    """
    Main pattern that apply for Episode, Volume, and Season.
    """
    _re_match_multi_number = re.compile(r"(\d+(?:\.\d+)?)(?:(v)(\d+))?"  # Episode number, release version
                                        r"([" + parser_helper.ESCAPED_DASHES +
                                        r" ~&+])"  # Range separator
                                        r"(\d+(?:\.\d+)?)(?:(v)(\d+))?$",  # Episode number, release version
                                        flags=re.IGNORECASE)
    _re_match_single_number = re.compile((r'^(\d+(?:\.\d+)?)'  # Episode number
                                          r'(?:(v)(\d+))?$'),  # Release version
                                         flags=re.IGNORECASE)

    def is_match_multi_numbers_pattern(self, token: Token, word: str, category: ElementCategory) -> bool:
        """
        Check if the word is a multi numbers pattern.
        e.g. "01-02", "01-02v2", "01v2-02v2", "01v2-02", "04.5-05.5", "04.5-05.5v2", "04.5v2-05.5v2", "04.5v2-05.5"
        pattern = <episode_number>[v<release_version>]<range_separator><episode_number>[v<release_version>]
        """
        match = self._re_match_multi_number.match(word)
        if match:
            lower_bound = match.group(1)
            upper_bound = match.group(5)
            # Avoid matching expressions such as "009-1" or "5-2"
            if int(lower_bound) < int(upper_bound):

                # In reverse order to keep the order of tokens
                if match.group(7):
                    self.insert_tokens(token, match.group(7), TokenCategory.IDENTIFIER,
                                       ElementCategory.RELEASE_VERSION)
                if match.group(6):
                    self.insert_tokens(token, match.group(6), TokenCategory.INVALID, ElementCategory.UNKNOWN)
                # match group 5 is the upper bound
                self.insert_tokens(token, upper_bound, TokenCategory.IDENTIFIER, category)
                # match group 4 is the range separator
                self.insert_tokens(token, match.group(4), TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
                if match.group(3):
                    self.insert_tokens(token, match.group(3), TokenCategory.IDENTIFIER,
                                       ElementCategory.RELEASE_VERSION)
                if match.group(2):
                    self.insert_tokens(token, match.group(2), TokenCategory.INVALID, ElementCategory.UNKNOWN)

                # match.group(1) is the lower bound
                self.set_token_element(token, TokenCategory.IDENTIFIER, category)
                token.content = lower_bound
                return True
        return False

    def is_match_number_patterns(self, token: Token, word: str, category: ElementCategory,
                                 prefix: bool = False) -> bool:
        """
        Check if the word have a valid numbered pattern.
        """
        if parser_helper.is_potential_number(word) and not prefix:
            # If the word only a number and the prefix is not set, then try to check the previous token
            previous_token = self.find_prev(token, TokenFlags.IDENTIFIER | TokenFlags.UNKNOWN)
            if not previous_token:
                return True
            if previous_token.content.lower() in ["part"]:
                return False
            if previous_token.t_category in [TokenCategory.DELIMITER, TokenCategory.BRACKET]:
                self.set_token_element(previous_token, TokenCategory.INVALID, ElementCategory.UNKNOWN)
                self.set_token_element(token, TokenCategory.IDENTIFIER, category)
                return True
            if parser_helper.is_dash_character(previous_token.content):
                self.set_token_element(token, TokenCategory.IDENTIFIER, category)
                return True
            keyword = self.keyword_manager.find(self.keyword_manager.normalize(previous_token.content))
            if keyword:
                if keyword.e_category in [ElementCategory.ANIME_SEASON, ElementCategory.ANIME_TYPE,
                                          ElementCategory.EPISODE_PREFIX]:
                    self.set_token_element(previous_token, TokenCategory.IDENTIFIER, keyword.e_category)
                    self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                elif keyword.e_category in [ElementCategory.VOLUME_PREFIX]:
                    self.set_token_element(previous_token, TokenCategory.IDENTIFIER, keyword.e_category)
                    self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.VOLUME_NUMBER)
                elif keyword.e_category in [ElementCategory.ANIME_SEASON_PREFIX]:
                    if not self.options['season_part_as_unique']:
                        self.set_token_element(previous_token, TokenCategory.IDENTIFIER, keyword.e_category)
                        self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_SEASON)
            # check if the next token is an unknown token and the content is "-"
            # if true, then it possibly an episode number
            next_token = self.find_next(token, TokenFlags.UNKNOWN | TokenFlags.IDENTIFIER)
            if next_token:
                if next_token.e_category in [
                    ElementCategory.AUDIO_TERM,
                    ElementCategory.VIDEO_TERM,
                    ElementCategory.VIDEO_RESOLUTION,
                    ElementCategory.EPISODE_TITLE,
                    ElementCategory.RELEASE_INFORMATION,
                    ElementCategory.RELEASE_VERSION,
                    ElementCategory.SOURCE,
                    ElementCategory.SUBTITLES,
                ]:
                    self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                    return True
                if previous_token == token and parser_helper.is_dash_character(next_token.content):
                    # found episode number-title separator
                    self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                    return True
            return False

        word = word.strip(' ' + parser_helper.DASHES)
        numeric_front = word[0].isdigit()
        numeric_back = word[-1].isdigit()

        # e.g. "01v2"
        if numeric_front and numeric_back:
            if self.is_match_single_number_pattern(token, word, category):
                return True
            if self.is_match_multi_numbers_pattern(token, word, category):
                return True
        if self.is_match_season_and_episode_pattern(token, word):
            return True
        # only valid for episode numbers
        if category == ElementCategory.EPISODE_NUMBER:
            if numeric_back:
                if self.is_match_number_sign_pattern(token, word):
                    return True
            if self.is_match_japanese_counter_pattern(token, word):
                return True
        if prefix:
            if self.is_match_partial_number_patterns(token, word):
                return True
        return False

    def is_match_partial_number_patterns(self, token: Token, word: str) -> bool:
        """
        Check if the word have a valid partial numbered pattern.
        e.g. "4a", "111C
        """
        non_number_begin = parser_helper.find_non_number_in_string(word)
        suffix = word[non_number_begin:]

        if len(suffix) == 1 and suffix.isalpha():
            token.content = word[:non_number_begin]
            self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)

            number = word[non_number_begin:]
            self.insert_tokens(token, number, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_PART)
            return True
        return False

    def is_match_single_number_pattern(self, token: Token, word: str, category: ElementCategory) -> bool:
        """
        Check if the word have a valid single numbered pattern.
        e.g. "01", "01v2", "07.5", "07.5v2"
        pattern = <episode_number>[v<release_version>]
        """
        match = self._re_match_single_number.match(word)
        if match:
            # in reverse order to keep the order of tokens
            if match.group(3):
                self.insert_tokens(token, match.group(3), TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION)
            if match.group(2):
                self.insert_tokens(token, match.group(2), TokenCategory.INVALID, ElementCategory.UNKNOWN)

            token.content = match.group(1)
            self.set_token_element(token, TokenCategory.IDENTIFIER, category)
            return True
        return False


class ParserNumber(Pattern):
    _re_search_buggy_dash = re.compile(r"(" + "|".join(parser_helper.DASHES) + r")",
                                       flags=re.IGNORECASE)

    def is_extent_keyword(self, token: Token, category: ElementCategory, prefix: bool) -> bool:
        next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)
        if next_token and next_token.t_category == TokenCategory.UNKNOWN:
            if parser_helper.find_number_in_string(next_token.content) is not None:
                # Check for episode, and volume number
                if category in [ElementCategory.EPISODE_NUMBER, ElementCategory.VOLUME_NUMBER]:
                    if self.is_match_number_patterns(next_token, next_token.content, category, prefix):
                        return True
        return False

    def is_number_comes_after_prefix(self, token: Token) -> bool:
        number_begin = parser_helper.find_number_in_string(token.content)
        prefix = token.content[:number_begin]

        keyword = self.keyword_manager.find(
            self.keyword_manager.normalize(prefix)
        )
        if keyword:
            if keyword.e_category not in [ElementCategory.EPISODE_PREFIX, ElementCategory.VOLUME_PREFIX,
                                          ElementCategory.ANIME_SEASON_PREFIX, ElementCategory.ANIME_TYPE]:
                return False

            number = token.content[number_begin:]
            if keyword.e_category == ElementCategory.EPISODE_PREFIX:
                category = ElementCategory.EPISODE_NUMBER
            elif keyword.e_category == ElementCategory.VOLUME_PREFIX:
                category = ElementCategory.VOLUME_NUMBER
            elif keyword.e_category == ElementCategory.ANIME_SEASON_PREFIX:
                category = ElementCategory.ANIME_SEASON
                additional_prefix = parser_helper.find_non_number_in_string(number)
                if additional_prefix:
                    left_over = number[additional_prefix:]
                    new_number_begin = parser_helper.find_number_in_string(left_over)
                    key = left_over[:new_number_begin]
                    lo_keyword = self.keyword_manager.find(
                        self.keyword_manager.normalize(key)
                    )
                    if (lo_keyword
                            and lo_keyword.e_category == ElementCategory.ANIME_TYPE):
                        number = number[:additional_prefix]
                    else:
                        if key.upper() == "S":
                            new_token = Token(prefix,
                                              TokenCategory.IDENTIFIER,
                                              ElementCategory.ANIME_SEASON_PREFIX,
                                              token.enclosed)
                            self.insert_before(token, new_token)
                            new_token = Token(number[:additional_prefix],
                                              TokenCategory.IDENTIFIER,
                                              ElementCategory.ANIME_SEASON,
                                              token.enclosed)
                            self.insert_before(token, new_token)
                            keyword = Keyword(ElementCategory.ANIME_TYPE, keyword.options)
                            category = ElementCategory.EPISODE_NUMBER
                            prefix = key
                            number = left_over[new_number_begin:]
                        del left_over
            elif keyword.e_category == ElementCategory.ANIME_TYPE:
                category = ElementCategory.EPISODE_NUMBER
            else:
                return False

            if self.is_match_number_patterns(token, number, category, True):
                new_token = Token(prefix, TokenCategory.IDENTIFIER, keyword.e_category, token.enclosed)
                self.insert_before(token, new_token)
                try:
                    left_token = Token(left_over, TokenCategory.UNKNOWN, ElementCategory.UNKNOWN, token.enclosed)
                    self.insert_after(token, left_token)
                    self.is_number_comes_after_prefix(left_token)
                except NameError:
                    pass
                return True
        else:
            number = token.content[number_begin:]
            if prefix and prefix[-1].lower() == "v" and number.isdigit():
                keyword = self.keyword_manager.find(
                    self.keyword_manager.normalize(prefix[:-1])
                )
                if keyword and keyword.e_category in [ElementCategory.EPISODE_PREFIX, ElementCategory.VOLUME_PREFIX,
                                                      ElementCategory.ANIME_SEASON_PREFIX, ElementCategory.ANIME_TYPE]:
                    token.content = prefix[:-1]
                    self.set_token_element(token, TokenCategory.IDENTIFIER, keyword.e_category)
                    new_token = Token(prefix[-1], TokenCategory.INVALID, ElementCategory.UNKNOWN, token.enclosed)
                    self.insert_after(token, new_token)
                    new_token = Token(number, TokenCategory.IDENTIFIER, ElementCategory.RELEASE_VERSION, token.enclosed)
                    self.insert_after(token, new_token)
                    return True
        return False

    def is_number_comes_before_another_number(self, token: Token) -> bool:
        """
        Check if the number comes before another number.
        """
        separator_token = self.find_next(token, TokenFlags.NOT_DELIMITER)

        if separator_token:
            separator = separator_token.content
            if separator == "&" or str(separator).lower() == "of":
                other_token = self.find_next(separator_token, TokenFlags.NOT_DELIMITER)
                if other_token and other_token.content.isdigit():
                    if separator == "&":
                        self.set_token_element(other_token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)

                    elif str(separator).lower() == "of":
                        self.set_token_element(other_token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_TOTAL)

                    self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                    self.set_token_element(separator_token, TokenCategory.INVALID, ElementCategory.RANGE_SEPARATOR)
                    return True
        return False

    def search_for_buggy_dash(self, tokens: List[Token]) -> None:
        """
        This is the last resort to fix the dash problem.
        e.g. "Gunbuster-ep1" -> "Gunbuster - ep1"

        :param tokens:
        :return:
        """
        for token in tokens:
            splitted = self._re_search_buggy_dash.split(token.content)
            placeholder = []
            new_token = Token("", token.t_category, token.e_category, token.enclosed)
            potential_tokens = []
            for text in splitted:
                if parser_helper.find_number_in_string(text) is not None:
                    if new_token.content:
                        # first and last dashes put on separate token
                        stripped_text = new_token.content.strip(parser_helper.DASHES)
                        dashes = new_token.content.split(stripped_text)
                        if dashes[0]:
                            placeholder.append(Token(dashes[0], TokenCategory.INVALID,
                                                     ElementCategory.UNKNOWN, token.enclosed))
                        new_token.content = stripped_text
                        placeholder.append(new_token)
                        if dashes[1]:
                            placeholder.append(Token(dashes[1], TokenCategory.INVALID,
                                                     ElementCategory.UNKNOWN, token.enclosed))

                        new_token = Token("", token.t_category, token.e_category, token.enclosed)
                    number_token = Token(text, token.t_category, token.e_category, token.enclosed)
                    placeholder.append(number_token)
                    potential_tokens.append(number_token)
                else:
                    new_token.content += text

            if new_token.content:
                stripped_text = new_token.content.strip(parser_helper.DASHES)
                dashes = new_token.content.split(stripped_text)
                if dashes[0]:
                    placeholder.append(Token(dashes[0], TokenCategory.INVALID,
                                             ElementCategory.UNKNOWN, token.enclosed))
                new_token.content = stripped_text
                placeholder.append(new_token)
                if dashes[1]:
                    placeholder.append(Token(dashes[1], TokenCategory.INVALID,
                                             ElementCategory.UNKNOWN, token.enclosed))

            if placeholder:
                placeholder.reverse()
                for i_token in placeholder:
                    self.insert_after(token, i_token)
                self.remove_token(token)

                is_found = False
                for potential_token in potential_tokens:
                    if self.is_number_comes_after_prefix(potential_token):
                        is_found = True
                    elif self.search_for_last_number([potential_token]):
                        is_found = True

                if not is_found:
                    self.insert_after(placeholder[-1], token)
                    for i_token in placeholder:
                        self.remove_token(i_token)
        return

    def search_for_equivalent_numbers(self, tokens: List[Token]) -> bool:
        """
        Search for equivalent numbers.
        """
        is_found = False
        for token in tokens:
            if self.is_token_isolated(token):
                continue

            previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)
            if previous_token and previous_token.e_category in [ElementCategory.ANIME_SEASON,
                                                                ElementCategory.ANIME_SEASON_PREFIX]:
                token.t_category = TokenCategory.IDENTIFIER
                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)

                is_found = True
                continue
            next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)
            if next_token is None or next_token.t_category != TokenCategory.UNKNOWN:
                continue

            # Check if it's an isolated number
            if not self.is_token_isolated(next_token):
                continue
            if not next_token.content.isdigit():
                continue

            episode = min(token, next_token, key=lambda t: t.content)
            continue_episode = max(token, next_token, key=lambda t: t.content)

            self.set_token_element(episode, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
            self.set_token_element(continue_episode, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
            is_found = True
        return is_found

    def search_for_isolated_numbers(self, tokens: List[Token]) -> bool:
        """
        Search for isolated numbers.
        """
        is_found = False
        for token in tokens:
            if not token.enclosed:
                continue
            if not self.is_token_isolated(token):
                continue
            if token.content.isdigit():
                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)

                is_found = True
        return is_found

    def search_for_last_number(self, tokens: List[Token]) -> bool:
        """
        Search for the last number.
        """
        is_found = False
        for token in tokens:
            token_index = self.get_index(token)

            # assuming the episode number always come after the title
            if token_index == 0:
                continue

            if token.enclosed:
                continue

            # Ignore if it's the first non-enclosed, non-delimiter token
            if all([t.enclosed or t.t_category == TokenCategory.DELIMITER
                    for t in self.get_list()[:token_index]]):
                continue

            # Ignore if the previous token is "Movie" or "Part"
            previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)
            if previous_token and str(previous_token.content).lower() in ["part"]:
                continue
            previous_token = self.find_prev(token, TokenFlags.UNKNOWN | TokenFlags.BRACKET)
            if not previous_token or previous_token == token:  # , because previous_token most likely same as token
                continue
            if not parser_helper.is_dash_character(previous_token.content):
                next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)
                if next_token and parser_helper.is_dash_character(next_token.content):
                    try:
                        f_text = float(token.content)
                        if f_text.is_integer():
                            if str(int(f_text)) != token.content:
                                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                                is_found = True
                        elif str(f_text) != token.content:
                            self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                            is_found = True
                    except ValueError:
                        pass

                continue
            if token.content.isdigit():
                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                is_found = True
        return is_found

    def search_for_number_patterns(self, tokens: List[Token]) -> bool:
        """
        Search for number patterns.
        """
        is_found = False
        for token in tokens:
            numeric_front = token.content[0].isdigit()

            if not numeric_front:
                # e.g. "EP.1", "Vol.1"
                if self.is_number_comes_after_prefix(token):
                    is_found = True
            elif self.is_number_comes_before_another_number(token):
                is_found = True

            if token.t_category == TokenCategory.UNKNOWN:
                # look for other patterns
                if is_found and token.content.upper() == "3D":
                    continue
                # if previous token is a dash, it is most likely an episode number
                prefix = False
                previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)
                if previous_token:
                    keyword = self.keyword_manager.find(
                        self.keyword_manager.normalize(previous_token.content))
                    prev_is_dash = parser_helper.is_dash_character(previous_token.content)
                else:
                    prev_is_dash = False

                if prev_is_dash:
                    prefix = True
                else:
                    # in case the separator not a dash,
                    # we need to check if the next token are the same with the previous token
                    next_token = self.find_next(token, TokenFlags.NOT_DELIMITER)

                    # the number are at the beginning of the string
                    if next_token:
                        next_is_dash = parser_helper.is_dash_character(next_token.content)
                        if (previous_token.content == next_token.content
                                or (previous_token == token and next_is_dash)
                                or (previous_token.t_category == next_token.t_category == TokenCategory.BRACKET
                                    and self.counter.get(ElementCategory.EPISODE_NUMBER))):
                            prefix = True
                    else:
                        # 02 is most likely an episode number
                        # but 2 is not guaranteed
                        try:
                            is_number = float(token.content)
                            if is_number.is_integer():
                                if str(int(is_number)) != token.content:
                                    prefix = True
                            else:
                                if str(is_number) != token.content:
                                    prefix = True
                        except ValueError:
                            # in case it 04v2, then it most likely an episode number
                            prefix = True

                    if not prefix:
                        if keyword and keyword.e_category == ElementCategory.ANIME_TYPE:
                            prefix = True

                if self.is_match_number_patterns(token, token.content, ElementCategory.EPISODE_NUMBER, prefix):
                    is_found = True
                    if prefix:
                        if keyword and keyword.e_category == ElementCategory.ANIME_TYPE:
                            self.set_token_element(previous_token, TokenCategory.IDENTIFIER, ElementCategory.ANIME_TYPE)
        return is_found

    def search_for_seperated_numbers(self, tokens: List[Token]) -> bool:
        """
        Search for seperated numbers.
        """
        is_found = False
        for token in tokens:
            previous_token = self.find_prev(token, TokenFlags.NOT_DELIMITER)

            if previous_token is None or previous_token.t_category != TokenCategory.UNKNOWN:
                continue

            if parser_helper.is_dash_character(previous_token.content) and token.content.isdigit():
                self.set_token_element(token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_NUMBER)
                self.set_token_element(previous_token, TokenCategory.IDENTIFIER, ElementCategory.EPISODE_PREFIX)

                is_found = True
        return is_found



import re

from aniparse import parser_helper
from aniparse.element import ElementCategory
from aniparse.parser import Parser
from aniparse.token import TokenCategory


class Aniparse(Parser):
    def __init__(self, filename, options, keywords_manager):
        self.anime: dict = {
            ElementCategory.FILE_NAME.value: filename
        }

        super().__init__(filename, options, keywords_manager)

    def parse(self) -> None:
        """
        Parse the filename, getting the elements and tokens
        :return:
        """
        self.remove_extension_from_filename()

        self.remove_ignored_strings_from_filename()

        if not self.filename:
            self.anime = None
            return  # the anime is unknown
        super().parse()

    def populate(self) -> dict:
        """
        Populate the anime dictionary with the elements and tokens
        :return:
        """
        for token in self.tokens:
            element = token.e_category
            if token.t_category == TokenCategory.INVALID:
                continue

            if token.t_category == TokenCategory.BRACKET:
                continue
            if token.t_category == TokenCategory.DELIMITER:
                continue

            if token.t_category == TokenCategory.UNKNOWN or element == ElementCategory.UNKNOWN:
                element = ElementCategory.OTHER

            if self.options["ignored_dash"]:
                if parser_helper.is_dash_character(token.content):
                    continue

            if element in [
                ElementCategory.ANIME_SEASON,
                ElementCategory.ANIME_YEAR,
                ElementCategory.EPISODE_NUMBER,
                ElementCategory.EPISODE_NUMBER_ALT,
                ElementCategory.EPISODE_TOTAL,
                ElementCategory.RELEASE_VERSION,
                ElementCategory.VOLUME_NUMBER
            ]:
                # set the content to be a number, or float if it has a decimal
                try:
                    token.content = int(token.content)
                except ValueError:
                    token.content = float(token.content)

            # if the element already in the dictionary, transform it into a list and append the new value
            element = element.value
            if element in self.anime:
                if not isinstance(self.anime[element], list):
                    self.anime[element] = [self.anime[element]]
                self.anime[element].append(token.content)
            else:
                self.anime[element] = token.content
        # clean up the anime dictionary
        for element in [ElementCategory.ANIME_TITLE,
                        ElementCategory.ANIME_TITLE_ALT,
                        ElementCategory.EPISODE_TITLE]:
            value = element.value
            if isinstance(self.anime.get(value, None), list):
                string = "".join(self.anime[value]).strip(
                    " " + parser_helper.DASHES if self.options["ignored_dash"] else "")  # any similar dash
            elif isinstance(self.anime.get(value, None), str):
                string = self.anime.get(value, "")
            else:
                continue
            # remove double spaces
            string = re.sub(r"\s+", " ", string)
            if string.isspace():
                self.anime.pop(value)
            else:
                self.anime[value] = string

        return self.anime

    def remove_extension_from_filename(self) -> None:
        split_filename = self.filename.rsplit('.', 1)

        if len(split_filename) < 2:
            return

        new_filename, extension = split_filename

        if len(extension) > self.options['max_extension_length']:
            return

        # normalize the extension and check if it is a valid extension
        keyword = self.keyword_manager.normalize(extension)
        # only return if the extension is a valid video extension
        if not self.keyword_manager.find(keyword, ElementCategory.FILE_EXTENSION):
            return

        if extension:
            self.anime[ElementCategory.FILE_EXTENSION.value] = extension
        self.filename = new_filename

    def remove_ignored_strings_from_filename(self) -> None:
        for string in self.options["ignored_strings"]:
            self.filename = self.filename.replace(string, '')




from unittest import TestCase

import aniparse
from aniparse.element import ElementCategory
from aniparse.keyword import KeywordManager, option
from tests.fixtures.table import table


class TestAniparse(TestCase):

    def test_table(self):
        keyword = KeywordManager()
        keyword.add(ElementCategory.RELEASE_GROUP, option["default"], [
            'THORA'
        ])
        for index, expected in enumerate(table):
            filename = expected["file_name"]
            anime = aniparse.parse(filename, keyword_manager=keyword)
            self.assertEqual(expected, anime, 'on entry number %d' % index)

    def test_season_part_as_unique(self):
        titles = {
            'Attack On Titan Season 3 Part 2.mkv': {
                True: {'file_name': 'Attack On Titan Season 3 Part 2.mkv',
                       'file_extension': 'mkv',
                       'anime_title': 'Attack On Titan Season 3 Part 2'},
                False: {'file_name': 'Attack On Titan Season 3 Part 2.mkv',
                        'file_extension': 'mkv',
                        'anime_title': 'Attack On Titan',
                        'anime_season_prefix': 'Season',
                        'anime_season': 3},
            },
            'Attack On Titan Season 3 Part 2 - 1.mkv': {
                True: {'file_name': 'Attack On Titan Season 3 Part 2 - 1.mkv',
                       'file_extension': 'mkv',
                       'anime_title': 'Attack On Titan Season 3 Part 2',
                       'episode_number': 1},
                False: {'file_name': 'Attack On Titan Season 3 Part 2 - 1.mkv',
                        'file_extension': 'mkv',
                        'anime_title': 'Attack On Titan',
                        'anime_season_prefix': 'Season',
                        'anime_season': 3,
                        'episode_number': 1},
            }
        }

        for title, options in titles.items():
            for condition, expected in options.items():
                anime = aniparse.parse(title, options={'season_part_as_unique': condition})
                self.assertEqual(expected, anime)



from setuptools import setup, find_packages

with open("README.md", encoding="utf-8") as f:
    README = f.read()

setup(
    name='aniparse',
    version='1.2.1',
    description="An anime video filename parser",
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/MeGaNeKoS/aniparse',
    license='License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
    author='めがねこ',
    author_email='evictory91+pypackages@gmail.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    packages=find_packages(),
    keywords=['anime', 'filename parser']
)
